# -*- coding: utf-8 -*-
##
## This file is part of Harvesting Kit.
## Copyright (C) 2014 CERN.
##
## Harvesting Kit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License as
## published by the Free Software Foundation; either version 2 of the
## License, or (at your option) any later version.
##
## Harvesting Kit is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Harvesting Kit; if not, write to the Free Software Foundation, Inc.,
## 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""
The BibRecord test suite.
"""

import unittest
import os
import pkg_resources

from harvestingkit import bibrecord


class BibRecordSuccessTest(unittest.TestCase):
    """ bibrecord - demo file parsing test """

    def setUp(self):
        """Initialize stuff"""
        xmltext = pkg_resources.resource_string('harvestingkit.tests',
                os.path.join('data', 'sample_bibrecord.xml'))
        self.recs = [rec[0] for rec in bibrecord.create_records(xmltext)]

    def test_records_created(self):
        """ bibrecord - demo file how many records are created """
        self.assertEqual(142, len(self.recs))

    def test_tags_created(self):
        """ bibrecord - demo file which tags are created """
        ## check if the tags are correct

        tags = ['003', '005', '020', '024', '035', '037', '041', '080', '084', '088',
                '100', '110', '148', '150', '242', '245', '246', '250', '260', '269',
                '270', '300', '340', '371', '372', '400', '410', '430', '440', '450',
                '490', '500', '502', '506', '510', '520', '542', '550', '588', '590',
                '595', '643', '650', '653', '670', '678', '680', '690', '691', '693',
                '694', '695', '697', '700', '710', '711', '720', '773', '852', '856',
                '859', '901', '909', '913', '914', '916', '920', '960', '961', '962',
                '963', '964', '970', '980', '999', 'FFT']

        t = []
        for rec in self.recs:
            t.extend(rec.keys())
        t.sort()
        #eliminate the elements repeated
        tt = []
        for x in t:
            if not x in tt:
                tt.append(x)
        self.assertEqual(tags, tt)

    def test_fields_created(self):
        """bibrecord - demo file how many fields are created"""
        ## check if the number of fields for each record is correct

        fields = [14, 14, 8, 11, 11, 13, 11, 15, 10, 18, 15, 16,
                  10, 9, 15, 10, 11, 11, 11, 9, 11, 11, 10, 9, 9, 9,
                  10, 9, 10, 10, 8, 9, 8, 9, 14, 13, 14, 14, 15, 12,
                  13, 12, 15, 15, 13, 16, 16, 15, 15, 14, 16, 15, 15,
                  15, 16, 15, 16, 15, 15, 16, 15, 15, 14, 15, 12, 13,
                  11, 15, 8, 11, 14, 13, 12, 13, 6, 6, 25, 24, 27, 26,
                  26, 24, 26, 26, 25, 28, 24, 23, 27, 25, 25, 26, 26,
                  25, 20, 26, 25, 22, 9, 8, 9, 9, 8, 7, 19, 21, 27, 23,
                  23, 22, 9, 8, 16, 7, 7, 9, 5, 5, 3, 9, 12, 6,
                  8, 8, 8, 13, 20, 20, 5, 8, 7, 7, 7, 7, 7, 8, 7, 8, 7, 7, 8, 6]
        cr = []
        ret = []
        for rec in self.recs:
            cr.append(len(rec.values()))
            ret.append(rec)
        self.assertEqual(fields, cr, "\n%s\n!=\n%s" % (fields, cr))

    def test_create_record_with_collection_tag(self):
        """ bibrecord - create_record() for single record in collection"""
        xmltext = """
        <collection>
        <record>
        <controlfield tag="001">33</controlfield>
        <datafield tag="041" ind1=" " ind2=" ">
        <subfield code="a">eng</subfield>
        </datafield>
        </record>
        </collection>
        """
        record = bibrecord.create_record(xmltext)
        record1 = bibrecord.create_records(xmltext)[0]
        self.assertEqual(record1, record)

class BibRecordParsersTest(unittest.TestCase):
    """ bibrecord - testing the creation of records with different parsers"""

    def setUp(self):
        """Initialize stuff"""
        self.xmltext = """
        <!-- A first comment -->
        <collection>
        <record>
        <controlfield tag="001">33</controlfield>
        <datafield tag="041" ind1=" " ind2=" ">
        <!-- A second comment -->
        <subfield code="a">eng</subfield>
        </datafield>
        </record>
        </collection>
        """
        self.expected_record = {
            '001': [([], ' ', ' ', '33', 1)],
            '041': [([('a', 'eng')], ' ', ' ', '', 2)]
            }

    def test_lxml(self):
        """ bibrecord - create_record() with lxml"""
        record = bibrecord._create_record_lxml(self.xmltext)
        self.assertEqual(record, self.expected_record)


class BibRecordDropDuplicateFieldsTest(unittest.TestCase):
    def test_drop_duplicate_fields(self):
        """bibrecord - testing record_drop_duplicate_fields()"""
        record = """
        <record>
        <controlfield tag="001">123</controlfield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="a">Doe, John</subfield>
        <subfield code="u">Foo University</subfield>
        </datafield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="a">Doe, John</subfield>
        <subfield code="u">Foo University</subfield>
        </datafield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="u">Foo University</subfield>
        <subfield code="a">Doe, John</subfield>
        </datafield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="a">Doe, John</subfield>
        <subfield code="u">Foo University</subfield>
        <subfield code="a">Doe, John</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2=" ">
        <subfield cde="a">On the foo and bar</subfield>
        </datafield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="a">Doe, John</subfield>
        <subfield code="u">Foo University</subfield>
        </datafield>
        </record>
        """
        record_result = """
        <record>
        <controlfield tag="001">123</controlfield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="a">Doe, John</subfield>
        <subfield code="u">Foo University</subfield>
        </datafield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="u">Foo University</subfield>
        <subfield code="a">Doe, John</subfield>
        </datafield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="a">Doe, John</subfield>
        <subfield code="u">Foo University</subfield>
        <subfield code="a">Doe, John</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2=" ">
        <subfield cde="a">On the foo and bar</subfield>
        </datafield>
        </record>
        """
        rec = bibrecord.create_record(record)[0]
        rec = bibrecord.record_drop_duplicate_fields(rec)
        rec2 = bibrecord.create_record(record_result)[0]
        self.maxDiff = None
        self.assertEqual(rec, rec2)


class BibRecordBadInputTreatmentTest(unittest.TestCase):
    """ bibrecord - testing for bad input treatment """
    def test_empty_collection(self):
        """bibrecord - empty collection"""
        xml_error0 = """<collection></collection>"""
        rec = bibrecord.create_record(xml_error0)[0]
        self.assertEqual(rec, {})
        records = bibrecord.create_records(xml_error0)
        self.assertEqual(len(records), 0)

    def test_wrong_attribute(self):
        """bibrecord - bad input subfield \'cde\' instead of \'code\'"""
        ws = bibrecord.CFG_BIBRECORD_WARNING_MSGS
        xml_error1 = """
        <record>
        <controlfield tag="001">33</controlfield>
        <datafield tag="041" ind1=" " ind2=" ">
        <subfield code="a">eng</subfield>
        </datafield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="a">Doe, John</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2=" ">
        <subfield cde="a">On the foo and bar</subfield>
        </datafield>
        </record>
        """
        e = bibrecord.create_record(xml_error1, 1, 1)[2]
        ee =''
        for i in e:
            if type(i).__name__ == 'str':
                if i.count(ws[3])>0:
                    ee = i
        self.assertEqual(bibrecord._warning((3, '(field number: 4)')), ee)

    def test_missing_attribute(self):
        """ bibrecord - bad input missing \"tag\" """
        ws = bibrecord.CFG_BIBRECORD_WARNING_MSGS
        xml_error2 = """
        <record>
        <controlfield tag="001">33</controlfield>
        <datafield ind1=" " ind2=" ">
        <subfield code="a">eng</subfield>
        </datafield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="a">Doe, John</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2=" ">
        <subfield code="a">On the foo and bar</subfield>
        </datafield>
        </record>
        """
        e = bibrecord.create_record(xml_error2, 1, 1)[2]
        ee = ''
        for i in e:
            if type(i).__name__ == 'str':
                if i.count(ws[1])>0:
                    ee = i
        self.assertEqual(bibrecord._warning((1, '(field number(s): [2])')), ee)

    def test_empty_datafield(self):
        """ bibrecord - bad input no subfield """
        ws = bibrecord.CFG_BIBRECORD_WARNING_MSGS
        xml_error3 = """
        <record>
        <controlfield tag="001">33</controlfield>
        <datafield tag="041" ind1=" " ind2=" ">
        </datafield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="a">Doe, John</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2=" ">
        <subfield code="a">On the foo and bar</subfield>
        </datafield>
        </record>
        """
        e = bibrecord.create_record(xml_error3, 1, 1)[2]
        ee = ''
        for i in e:
            if type(i).__name__ == 'str':
                if i.count(ws[8])>0:
                    ee = i
        self.assertEqual(bibrecord._warning((8, '(field number: 2)')), ee)

    def test_missing_tag(self):
        """bibrecord - bad input missing end \"tag\" """
        ws = bibrecord.CFG_BIBRECORD_WARNING_MSGS
        xml_error4 = """
        <record>
        <controlfield tag="001">33</controlfield>
        <datafield tag="041" ind1=" " ind2=" ">
        <subfield code="a">eng</subfield>
        </datafield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="a">Doe, John</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2=" ">
        <subfield code="a">On the foo and bar</subfield>
        </record>
        """
        e = bibrecord.create_record(xml_error4, 1, 1)[2]
        ee = ''
        for i in e:
            if type(i).__name__ == 'str':
                if i.count(ws[99])>0:
                    ee = i
        self.assertEqual(bibrecord._warning((99, '(Tagname : datafield)')), ee)

class BibRecordAccentedUnicodeLettersTest(unittest.TestCase):
    """ bibrecord - testing accented UTF-8 letters """

    def setUp(self):
        """Initialize stuff"""
        self.xml_example_record = """<record>
  <controlfield tag="001">33</controlfield>
  <datafield tag="041" ind1=" " ind2=" ">
    <subfield code="a">eng</subfield>
  </datafield>
  <datafield tag="100" ind1=" " ind2=" ">
    <subfield code="a">Döè1, John</subfield>
  </datafield>
  <datafield tag="100" ind1=" " ind2=" ">
    <subfield code="a">Doe2, J>ohn</subfield>
    <subfield code="b">editor</subfield>
  </datafield>
  <datafield tag="245" ind1=" " ind2="1">
    <subfield code="a">Пушкин</subfield>
  </datafield>
  <datafield tag="245" ind1=" " ind2="2">
    <subfield code="a">On the foo and bar2</subfield>
  </datafield>
</record>"""
        self.rec = bibrecord.create_record(self.xml_example_record, 1, 1)[0]

    def test_accented_unicode_characters(self):
        """bibrecord - accented Unicode letters"""
        self.assertEqual(self.xml_example_record,
                         bibrecord.record_xml_output(self.rec))
        self.assertEqual(bibrecord.record_get_field_instances(self.rec, "100", " ", " "),
                         [([('a', 'Döè1, John')], " ", " ", "", 3), ([('a', 'Doe2, J>ohn'), ('b', 'editor')], " ", " ", "", 4)])
        self.assertEqual(bibrecord.record_get_field_instances(self.rec, "245", " ", "1"),
                         [([('a', 'Пушкин')], " ", '1', "", 5)])

class BibRecordGettingFieldValuesTest(unittest.TestCase):
    """ bibrecord - testing for getting field/subfield values """

    def setUp(self):
        """Initialize stuff"""
        xml_example_record = """
        <record>
        <controlfield tag="001">33</controlfield>
        <datafield tag="041" ind1=" " ind2=" ">
        <subfield code="a">eng</subfield>
        </datafield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="a">Doe1, John</subfield>
        </datafield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="a">Doe2, John</subfield>
        <subfield code="b">editor</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2="1">
        <subfield code="a">On the foo and bar1</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2="2">
        <subfield code="a">On the foo and bar2</subfield>
        </datafield>
        <datafield tag="700" ind1=" " ind2="2">
        <subfield code="a">Penrose, Roger</subfield>
        <subfield code="u">University College London</subfield>
        </datafield>
        <datafield tag="700" ind1=" " ind2="2">
        <subfield code="a">Messi, Lionel</subfield>
        <subfield code="u">FC Barcelona</subfield>
        </datafield>
        </record>
        """
        self.rec = bibrecord.create_record(xml_example_record, 1, 1)[0]

    def test_get_field_instances(self):
        """bibrecord - getting field instances"""
        self.assertEqual(bibrecord.record_get_field_instances(self.rec, "100", " ", " "),
                         [([('a', 'Doe1, John')], " ", " ", "", 3), ([('a', 'Doe2, John'), ('b', 'editor')], " ", " ", "", 4)])
        self.assertEqual(bibrecord.record_get_field_instances(self.rec, "", " ", " "),
                        [('245', [([('a', 'On the foo and bar1')], " ", '1', "", 5), ([('a', 'On the foo and bar2')], " ", '2', "", 6)]), ('001', [([], " ", " ", '33', 1)]), ('700', [([('a', 'Penrose, Roger'), ('u', "University College London")], ' ', '2', '', 7), ([('a', 'Messi, Lionel'), ('u', 'FC Barcelona')], ' ', '2', '', 8)]), ('100', [([('a', 'Doe1, John')], " ", " ", "", 3), ([('a', 'Doe2, John'), ('b', 'editor')], " ", " ", "", 4)]), ('041', [([('a', 'eng')], " ", " ", "", 2)]),])

    def test_get_field_values(self):
        """bibrecord - getting field values"""
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "100", " ", " ", "a"),
                         ['Doe1, John', 'Doe2, John'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "100", " ", " ", "b"),
                         ['editor'])

    def test_get_field_value(self):
        """bibrecord - getting first field value"""
        self.assertEqual(bibrecord.record_get_field_value(self.rec, "100", " ", " ", "a"),
                         'Doe1, John')
        self.assertEqual(bibrecord.record_get_field_value(self.rec, "100", " ", " ", "b"),
                         'editor')

    def test_get_subfield_values(self):
        """bibrecord - getting subfield values"""
        fi1, fi2 = bibrecord.record_get_field_instances(self.rec, "100", " ", " ")
        self.assertEqual(bibrecord.field_get_subfield_values(fi1, "b"), [])
        self.assertEqual(bibrecord.field_get_subfield_values(fi2, "b"), ["editor"])

    def test_filter_field(self):
        """bibrecord - filter field instances"""
        field_instances = bibrecord.record_get_field_instances(self.rec, "700", "%", "%")
        out = bibrecord.filter_field_instances(field_instances, "u", "University College London", 'e')
        self.assertEqual(out, [([('a', 'Penrose, Roger'), ('u', "University College London")], ' ', '2', '', 7)])
        out = bibrecord.filter_field_instances(field_instances, "u", "Bar", "s")
        self.assertEqual(out, [([('a', 'Messi, Lionel'), ('u', 'FC Barcelona')], ' ', '2', '', 8)])
        out = bibrecord.filter_field_instances(field_instances, "u", "on", "s")
        self.assertEqual(out, [([('a', 'Penrose, Roger'), ('u', "University College London")], ' ', '2', '', 7),
                               ([('a', 'Messi, Lionel'), ('u', 'FC Barcelona')], ' ', '2', '', 8)])
        out = bibrecord.filter_field_instances(field_instances, "u", r".*\scoll", "r")
        self.assertEqual(out,[])
        out = bibrecord.filter_field_instances(field_instances, "u", r"[FC]{2}\s.*", "r")
        self.assertEqual(out, [([('a', 'Messi, Lionel'), ('u', 'FC Barcelona')], ' ', '2', '', 8)])


class BibRecordGettingFieldValuesViaWildcardsTest(unittest.TestCase):
    """ bibrecord - testing for getting field/subfield values via wildcards """

    def setUp(self):
        """Initialize stuff"""
        xml_example_record = """
        <record>
        <controlfield tag="001">1</controlfield>
        <datafield tag="100" ind1="C" ind2="5">
        <subfield code="a">val1</subfield>
        </datafield>
        <datafield tag="555" ind1="A" ind2="B">
        <subfield code="a">val2</subfield>
        </datafield>
        <datafield tag="555" ind1="A" ind2=" ">
        <subfield code="a">val3</subfield>
        </datafield>
        <datafield tag="555" ind1=" " ind2=" ">
        <subfield code="a">val4a</subfield>
        <subfield code="b">val4b</subfield>
        </datafield>
        <datafield tag="555" ind1=" " ind2="B">
        <subfield code="a">val5</subfield>
        </datafield>
        <datafield tag="556" ind1="A" ind2="C">
        <subfield code="a">val6</subfield>
        </datafield>
        <datafield tag="556" ind1="A" ind2=" ">
        <subfield code="a">val7a</subfield>
        <subfield code="b">val7b</subfield>
        </datafield>
        </record>
        """
        self.rec = bibrecord.create_record(xml_example_record, 1, 1)[0]

    def test_get_field_instances_via_wildcard(self):
        """bibrecord - getting field instances via wildcards"""
        self.assertEqual(bibrecord.record_get_field_instances(self.rec, "100", " ", " "),
                         [])
        self.assertEqual(bibrecord.record_get_field_instances(self.rec, "100", "%", " "),
                         [])
        self.assertEqual(bibrecord.record_get_field_instances(self.rec, "100", "%", "%"),
                         [([('a', 'val1')], 'C', '5', "", 2)])
        self.assertEqual(bibrecord.record_get_field_instances(self.rec, "55%", "A", "%"),
                         [([('a', 'val2')], 'A', 'B', "", 3),
                          ([('a', 'val3')], 'A', " ", "", 4),
                          ([('a', 'val6')], 'A', 'C', "", 7),
                          ([('a', 'val7a'), ('b', 'val7b')], 'A', " ", "", 8)])
        self.assertEqual(bibrecord.record_get_field_instances(self.rec, "55%", "A", " "),
                         [([('a', 'val3')], 'A', " ", "", 4),
                          ([('a', 'val7a'), ('b', 'val7b')], 'A', " ", "", 8)])
        self.assertEqual(bibrecord.record_get_field_instances(self.rec, "556", "A", " "),
                         [([('a', 'val7a'), ('b', 'val7b')], 'A', " ", "", 8)])

    def test_get_field_values_via_wildcard(self):
        """bibrecord - getting field values via wildcards"""
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "100", " ", " ", " "),
                         [])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "100", "%", " ", " "),
                         [])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "100", " ", "%", " "),
                         [])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "100", "%", "%", " "),
                         [])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "100", "%", "%", "z"),
                         [])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "100", " ", " ", "%"),
                         [])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "100", " ", " ", "a"),
                        [])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "100", "%", " ", "a"),
                         [])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "100", "%", "%", "a"),
                         ['val1'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "100", "%", "%", "%"),
                         ['val1'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "55%", "A", "%", "a"),
                         ['val2', 'val3', 'val6', 'val7a'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "55%", "A", " ", "a"),
                         ['val3', 'val7a'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "556", "A", " ", "a"),
                         ['val7a'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "555", " ", " ", " "),
                         [])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "555", " ", " ", "z"),
                         [])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "555", " ", " ", "%"),
                         ['val4a', 'val4b'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "55%", " ", " ", "b"),
                         ['val4b'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "55%", "%", "%", "b"),
                         ['val4b', 'val7b'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "55%", "A", " ", "b"),
                         ['val7b'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "55%", "A", "%", "b"),
                         ['val7b'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "55%", "A", " ", "a"),
                         ['val3', 'val7a'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "55%", "A", "%", "a"),
                         ['val2', 'val3', 'val6', 'val7a'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "55%", "%", "%", "a"),
                         ['val2', 'val3', 'val4a', 'val5', 'val6', 'val7a'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "55%", " ", " ", "a"),
                         ['val4a'])

    def test_get_field_values_filtering_exact(self):
        """bibrecord - getting field values and exact filtering"""
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "556", "%", "%", "%", 'a', 'val7a'),
                         ['val7a', 'val7b'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "556", "%", "%", "a", 'a', 'val7a'),
                         ['val7a'])

    def test_get_field_values_filtering_substring(self):
        """bibrecord - getting field values and substring filtering"""
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "556", "%", "%", "%", 'a', '7a', 's'),
                         ['val7a', 'val7b'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "556", "%", "%", "b", 'a', '7a', 's'),
                         ['val7b'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "55%", "%", "%", "%", 'b', 'val', 's'),
                         ['val4a', 'val4b', 'val7a', 'val7b'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "55%", "%", "%", " ", 'b', 'val', 's'),
                         [])

    def test_get_field_values_filtering_regexp(self):
        """bibrecord - getting field values and regexp filtering"""
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "55%", "%", "%", "%", 'b', r'al', 'r'),
                         [])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "55%", "%", "%", "%", 'a', r'.*al[6,7]', 'r'),
                         ['val6', 'val7a', 'val7b'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "55%", "%", "%", "a", 'a', r'.*al[6,7]', 'r'),
                         ['val6', 'val7a'])

    def test_get_field_value_via_wildcard(self):
        """bibrecord - getting first field value via wildcards"""
        self.assertEqual(bibrecord.record_get_field_value(self.rec, "100", " ", " ", " "),
                         '')
        self.assertEqual(bibrecord.record_get_field_value(self.rec, "100", "%", " ", " "),
                         '')
        self.assertEqual(bibrecord.record_get_field_value(self.rec, "100", " ", "%", " "),
                         '')
        self.assertEqual(bibrecord.record_get_field_value(self.rec, "100", "%", "%", " "),
                         '')
        self.assertEqual(bibrecord.record_get_field_value(self.rec, "100", " ", " ", "%"),
                         '')
        self.assertEqual(bibrecord.record_get_field_value(self.rec, "100", " ", " ", "a"),
                         '')
        self.assertEqual(bibrecord.record_get_field_value(self.rec, "100", "%", " ", "a"),
                         '')
        self.assertEqual(bibrecord.record_get_field_value(self.rec, "100", "%", "%", "a"),
                         'val1')
        self.assertEqual(bibrecord.record_get_field_value(self.rec, "100", "%", "%", "%"),
                         'val1')
        self.assertEqual(bibrecord.record_get_field_value(self.rec, "55%", "A", "%", "a"),
                         'val2')
        self.assertEqual(bibrecord.record_get_field_value(self.rec, "55%", "A", " ", "a"),
                         'val3')
        self.assertEqual(bibrecord.record_get_field_value(self.rec, "556", "A", " ", "a"),
                         'val7a')
        self.assertEqual(bibrecord.record_get_field_value(self.rec, "555", " ", " ", " "),
                         '')
        self.assertEqual(bibrecord.record_get_field_value(self.rec, "555", " ", " ", "%"),
                         'val4a')
        self.assertEqual(bibrecord.record_get_field_value(self.rec, "55%", " ", " ", "b"),
                         'val4b')
        self.assertEqual(bibrecord.record_get_field_value(self.rec, "55%", "%", "%", "b"),
                         'val4b')
        self.assertEqual(bibrecord.record_get_field_value(self.rec, "55%", "A", " ", "b"),
                         'val7b')
        self.assertEqual(bibrecord.record_get_field_value(self.rec, "55%", "A", "%", "b"),
                         'val7b')
        self.assertEqual(bibrecord.record_get_field_value(self.rec, "55%", "A", " ", "a"),
                         'val3')
        self.assertEqual(bibrecord.record_get_field_value(self.rec, "55%", "A", "%", "a"),
                         'val2')
        self.assertEqual(bibrecord.record_get_field_value(self.rec, "55%", "%", "%", "a"),
                         'val2')
        self.assertEqual(bibrecord.record_get_field_value(self.rec, "55%", " ", " ", "a"),
                         'val4a')

class BibRecordAddFieldTest(unittest.TestCase):
    """ bibrecord - testing adding field """

    def setUp(self):
        """Initialize stuff"""
        xml_example_record = """
        <record>
        <controlfield tag="001">33</controlfield>
        <datafield tag="041" ind1=" " ind2=" ">
        <subfield code="a">eng</subfield>
        </datafield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="a">Doe1, John</subfield>
        </datafield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="a">Doe2, John</subfield>
        <subfield code="b">editor</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2="1">
        <subfield code="a">On the foo and bar1</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2="2">
        <subfield code="a">On the foo and bar2</subfield>
        </datafield>
        </record>
        """
        self.rec = bibrecord.create_record(xml_example_record, 1, 1)[0]

    def test_add_controlfield(self):
        """bibrecord - adding controlfield"""
        field_position_global_1 = bibrecord.record_add_field(self.rec, "003",
                                                    controlfield_value="SzGeCERN")
        field_position_global_2 = bibrecord.record_add_field(self.rec, "004",
                                                    controlfield_value="Test")
        self.assertEqual(field_position_global_1, 2)
        self.assertEqual(field_position_global_2, 3)
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "003", " ", " ", ""),
                         ['SzGeCERN'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "004", " ", " ", ""),
                         ['Test'])

    def test_add_datafield(self):
        """bibrecord - adding datafield"""
        field_position_global_1 = bibrecord.record_add_field(self.rec, "100",
                                                    subfields=[('a', 'Doe3, John')])
        field_position_global_2 = bibrecord.record_add_field(self.rec, "100",
                                                    subfields= [('a', 'Doe4, John'), ('b', 'editor')])
        self.assertEqual(field_position_global_1, 5)
        self.assertEqual(field_position_global_2, 6)
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "100", " ", " ", "a"),
                         ['Doe1, John', 'Doe2, John', 'Doe3, John', 'Doe4, John'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "100", " ", " ", "b"),
                         ['editor', 'editor'])

    def test_add_controlfield_on_desired_position(self):
        """bibrecord - adding controlfield on desired position"""
        field_position_global_1 = bibrecord.record_add_field(self.rec, "005",
                                                    controlfield_value="Foo",
                                                    field_position_global=0)
        field_position_global_2 = bibrecord.record_add_field(self.rec, "006",
                                                    controlfield_value="Bar",
                                                    field_position_global=0)
        self.assertEqual(field_position_global_1, 7)
        self.assertEqual(field_position_global_2, 8)

    def test_add_datafield_on_desired_position_field_position_global(self):
        """bibrecord - adding datafield on desired global field position"""
        field_position_global_1 = bibrecord.record_add_field(self.rec, "100",
            subfields=[('a', 'Doe3, John')], field_position_global=0)
        field_position_global_2 = bibrecord.record_add_field(self.rec, "100",
            subfields=[('a', 'Doe4, John'), ('b', 'editor')], field_position_global=0)
        self.assertEqual(field_position_global_1, 3)
        self.assertEqual(field_position_global_2, 3)

    def test_add_datafield_on_desired_position_field_position_local(self):
        """bibrecord - adding datafield on desired local field position"""
        field_position_global_1 = bibrecord.record_add_field(self.rec, "100",
             subfields=[('a', 'Doe3, John')], field_position_local=0)
        field_position_global_2 = bibrecord.record_add_field(self.rec, "100",
             subfields=[('a', 'Doe4, John'), ('b', 'editor')],
             field_position_local=2)
        self.assertEqual(field_position_global_1, 3)
        self.assertEqual(field_position_global_2, 5)

class BibRecordManageMultipleFieldsTest(unittest.TestCase):
    """ bibrecord - testing the management of multiple fields """

    def setUp(self):
        """Initialize stuff"""
        xml_example_record = """
        <record>
        <controlfield tag="001">33</controlfield>
        <datafield tag="245" ind1=" " ind2=" ">
        <subfield code="a">subfield1</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2=" ">
        <subfield code="a">subfield2</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2=" ">
        <subfield code="a">subfield3</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2=" ">
        <subfield code="a">subfield4</subfield>
        </datafield>
        </record>
        """
        self.rec = bibrecord.create_record(xml_example_record, 1, 1)[0]

    def test_delete_multiple_datafields(self):
        """bibrecord - deleting multiple datafields"""
        self.fields = bibrecord.record_delete_fields(self.rec, '245', [1, 2])
        self.assertEqual(self.fields[0],
            ([('a', 'subfield2')], ' ', ' ', '', 3))
        self.assertEqual(self.fields[1],
            ([('a', 'subfield3')], ' ', ' ', '', 4))

    def test_add_multiple_datafields_default_index(self):
        """bibrecord - adding multiple fields with the default index"""
        fields = [([('a', 'subfield5')], ' ', ' ', '', 4),
            ([('a', 'subfield6')], ' ', ' ', '', 19)]
        index = bibrecord.record_add_fields(self.rec, '245', fields)
        self.assertEqual(index, None)
        self.assertEqual(self.rec['245'][-2],
            ([('a', 'subfield5')], ' ', ' ', '', 6))
        self.assertEqual(self.rec['245'][-1],
            ([('a', 'subfield6')], ' ', ' ', '', 7))

    def test_add_multiple_datafields_with_index(self):
        """bibrecord - adding multiple fields with an index"""
        fields = [([('a', 'subfield5')], ' ', ' ', '', 4),
            ([('a', 'subfield6')], ' ', ' ', '', 19)]
        index = bibrecord.record_add_fields(self.rec, '245', fields,
            field_position_local=0)
        self.assertEqual(index, 0)
        self.assertEqual(self.rec['245'][0],
            ([('a', 'subfield5')], ' ', ' ', '', 2))
        self.assertEqual(self.rec['245'][1],
            ([('a', 'subfield6')], ' ', ' ', '', 3))
        self.assertEqual(self.rec['245'][2],
            ([('a', 'subfield1')], ' ', ' ', '', 4))

    def test_move_multiple_fields(self):
        """bibrecord - move multiple fields"""
        bibrecord.record_move_fields(self.rec, '245', [1, 3])
        self.assertEqual(self.rec['245'][0],
            ([('a', 'subfield1')], ' ', ' ', '', 2))
        self.assertEqual(self.rec['245'][1],
            ([('a', 'subfield3')], ' ', ' ', '', 4))
        self.assertEqual(self.rec['245'][2],
            ([('a', 'subfield2')], ' ', ' ', '', 5))
        self.assertEqual(self.rec['245'][3],
            ([('a', 'subfield4')], ' ', ' ', '', 6))

class BibRecordDeleteFieldTest(unittest.TestCase):
    """ bibrecord - testing field deletion """

    def setUp(self):
        """Initialize stuff"""
        xml_example_record = """
        <record>
        <controlfield tag="001">33</controlfield>
        <datafield tag="041" ind1=" " ind2=" ">
        <subfield code="a">eng</subfield>
        </datafield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="a">Doe1, John</subfield>
        </datafield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="a">Doe2, John</subfield>
        <subfield code="b">editor</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2="1">
        <subfield code="a">On the foo and bar1</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2="2">
        <subfield code="a">On the foo and bar2</subfield>
        </datafield>
        </record>
        """
        self.rec = bibrecord.create_record(xml_example_record, 1, 1)[0]

        xml_example_record_empty = """
        <record>
        </record>
        """
        self.rec_empty = bibrecord.create_record(xml_example_record_empty, 1, 1)[0]

    def test_delete_controlfield(self):
        """bibrecord - deleting controlfield"""
        bibrecord.record_delete_field(self.rec, "001", " ", " ")
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "001", " ", " ", " "),
                         [])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "100", " ", " ", "b"),
                         ['editor'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "245", " ", "2", "a"),
                         ['On the foo and bar2'])

    def test_delete_datafield(self):
        """bibrecord - deleting datafield"""
        bibrecord.record_delete_field(self.rec, "100", " ", " ")
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "001", " ", " ", ""),
                         ['33'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "100", " ", " ", "b"),
                         [])
        bibrecord.record_delete_field(self.rec, "245", " ", " ")
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "245", " ", "1", "a"),
                         ['On the foo and bar1'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "245", " ", "2", "a"),
                         ['On the foo and bar2'])
        bibrecord.record_delete_field(self.rec, "245", " ", "2")
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "245", " ", "1", "a"),
                         ['On the foo and bar1'])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "245", " ", "2", "a"),
                         [])

    def test_add_delete_add_field_to_empty_record(self):
        """bibrecord - adding, deleting, and adding back a field to an empty record"""
        field_position_global_1 = bibrecord.record_add_field(self.rec_empty, "003",
                                                    controlfield_value="SzGeCERN")
        self.assertEqual(field_position_global_1, 1)
        self.assertEqual(bibrecord.record_get_field_values(self.rec_empty, "003", " ", " ", ""),
                         ['SzGeCERN'])
        bibrecord.record_delete_field(self.rec_empty, "003", " ", " ")
        self.assertEqual(bibrecord.record_get_field_values(self.rec_empty, "003", " ", " ", ""),
                         [])
        field_position_global_1 = bibrecord.record_add_field(self.rec_empty, "003",
                                                    controlfield_value="SzGeCERN2")
        self.assertEqual(field_position_global_1, 1)
        self.assertEqual(bibrecord.record_get_field_values(self.rec_empty, "003", " ", " ", ""),
                         ['SzGeCERN2'])


class BibRecordDeleteFieldFromTest(unittest.TestCase):
    """ bibrecord - testing field deletion from position"""

    def setUp(self):
        """Initialize stuff"""
        xml_example_record = """
        <record>
        <controlfield tag="001">33</controlfield>
        <datafield tag="041" ind1=" " ind2=" ">
        <subfield code="a">eng</subfield>
        </datafield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="a">Doe1, John</subfield>
        </datafield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="a">Doe2, John</subfield>
        <subfield code="b">editor</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2="1">
        <subfield code="a">On the foo and bar1</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2="2">
        <subfield code="a">On the foo and bar2</subfield>
        </datafield>
        </record>
        """
        self.rec = bibrecord.create_record(xml_example_record, 1, 1)[0]

    def test_delete_field_from(self):
        """bibrecord - deleting field from position"""
        bibrecord.record_delete_field(self.rec, "100", field_position_global=4)
        self.assertEqual(self.rec['100'], [([('a', 'Doe1, John')], ' ', ' ', '', 3)])
        bibrecord.record_delete_field(self.rec, "100", field_position_global=3)
        self.failIf('100' in self.rec)
        bibrecord.record_delete_field(self.rec, "001", field_position_global=1)
        bibrecord.record_delete_field(self.rec, "245", field_position_global=6)
        self.failIf('001' in self.rec)
        self.assertEqual(self.rec['245'], [([('a', 'On the foo and bar1')], ' ', '1', '', 5)])

        # Some crash tests
        bibrecord.record_delete_field(self.rec, '999', field_position_global=1)
        bibrecord.record_delete_field(self.rec, '245', field_position_global=999)


class BibRecordAddSubfieldIntoTest(unittest.TestCase):
    """ bibrecord - testing subfield addition """

    def setUp(self):
        """Initialize stuff"""
        xml_example_record = """
        <record>
        <controlfield tag="001">33</controlfield>
        <datafield tag="041" ind1=" " ind2=" ">
        <subfield code="a">eng</subfield>
        </datafield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="a">Doe2, John</subfield>
        <subfield code="b">editor</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2="1">
        <subfield code="a">On the foo and bar1</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2="2">
        <subfield code="a">On the foo and bar2</subfield>
        </datafield>
        </record>
        """
        self.rec = bibrecord.create_record(xml_example_record, 1, 1)[0]

    def test_add_subfield_into(self):
        """bibrecord - adding subfield into position"""
        bibrecord.record_add_subfield_into(self.rec, "100", "b", "Samekniv",
            field_position_global=3)
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "100", " ", " ", "b"),
                         ['editor', 'Samekniv'])
        bibrecord.record_add_subfield_into(self.rec, "245", "x", "Elgokse",
            field_position_global=4)
        bibrecord.record_add_subfield_into(self.rec, "245", "x", "Fiskeflue",
            subfield_position=0, field_position_global=4)
        bibrecord.record_add_subfield_into(self.rec, "245", "z", "Ulriken",
            subfield_position=2, field_position_global=4)
        bibrecord.record_add_subfield_into(self.rec, "245", "z",
            "Stortinget", subfield_position=999, field_position_global=4)
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "245", " ", "1", "%"),
                         ['Fiskeflue', 'On the foo and bar1', 'Ulriken', 'Elgokse', 'Stortinget'])
        # Some crash tests
        self.assertRaises(bibrecord.InvenioBibRecordFieldError,
            bibrecord.record_add_subfield_into, self.rec, "187", "x", "Crash",
            field_position_global=1)
        self.assertRaises(bibrecord.InvenioBibRecordFieldError,
            bibrecord.record_add_subfield_into, self.rec, "245", "x", "Crash",
            field_position_global=999)


class BibRecordModifyControlfieldTest(unittest.TestCase):
    """ bibrecord - testing controlfield modification """

    def setUp(self):
        """Initialize stuff"""
        xml_example_record = """
        <record>
        <controlfield tag="001">33</controlfield>
        <controlfield tag="005">A Foo's Tale</controlfield>
        <controlfield tag="008">Skeech Skeech</controlfield>
        <controlfield tag="008">Whoop Whoop</controlfield>
        <datafield tag="041" ind1=" " ind2=" ">
        <subfield code="a">eng</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2="2">
        <subfield code="a">On the foo and bar2</subfield>
        </datafield>
        </record>
        """
        self.rec = bibrecord.create_record(xml_example_record, 1, 1)[0]

    def test_modify_controlfield(self):
        """bibrecord - modify controlfield"""
        bibrecord.record_modify_controlfield(self.rec, "001", "34",
            field_position_global=1)
        bibrecord.record_modify_controlfield(self.rec, "008", "Foo Foo",
            field_position_global=3)
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "001"), ["34"])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "005"), ["A Foo's Tale"])
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "008"), ["Foo Foo", "Whoop Whoop"])
        # Some crash tests
        self.assertRaises(bibrecord.InvenioBibRecordFieldError,
            bibrecord.record_modify_controlfield, self.rec, "187", "Crash",
            field_position_global=1)
        self.assertRaises(bibrecord.InvenioBibRecordFieldError,
            bibrecord.record_modify_controlfield, self.rec, "008", "Test",
            field_position_global=10)
        self.assertRaises(bibrecord.InvenioBibRecordFieldError,
            bibrecord.record_modify_controlfield, self.rec, "245", "Burn",
            field_position_global=5)
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "245", " ", "2", "%"),
                         ["On the foo and bar2"])


class BibRecordModifySubfieldTest(unittest.TestCase):
    """ bibrecord - testing subfield modification """

    def setUp(self):
        """Initialize stuff"""
        xml_example_record = """
        <record>
        <controlfield tag="001">33</controlfield>
        <datafield tag="041" ind1=" " ind2=" ">
        <subfield code="a">eng</subfield>
        </datafield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="a">Doe2, John</subfield>
        <subfield code="b">editor</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2="1">
        <subfield code="a">On the foo and bar1</subfield>
        <subfield code="b">On writing unit tests</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2="2">
        <subfield code="a">On the foo and bar2</subfield>
        </datafield>
        </record>
        """
        self.rec = bibrecord.create_record(xml_example_record, 1, 1)[0]

    def test_modify_subfield(self):
        """bibrecord - modify subfield"""
        bibrecord.record_modify_subfield(self.rec, "245", "a", "Holmenkollen",
            0, field_position_global=4)
        bibrecord.record_modify_subfield(self.rec, "245", "x", "Brann", 1,
            field_position_global=4)
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "245", " ", "1", "%"),
                         ['Holmenkollen', 'Brann'])
        # Some crash tests
        self.assertRaises(bibrecord.InvenioBibRecordFieldError,
            bibrecord.record_modify_subfield, self.rec, "187", "x", "Crash", 0,
            field_position_global=1)
        self.assertRaises(bibrecord.InvenioBibRecordFieldError,
            bibrecord.record_modify_subfield, self.rec, "245", "x", "Burn", 1,
            field_position_global=999)
        self.assertRaises(bibrecord.InvenioBibRecordFieldError,
            bibrecord.record_modify_subfield, self.rec, "245", "a", "Burn",
            999, field_position_global=4)

class BibRecordDeleteSubfieldFromTest(unittest.TestCase):
    """ bibrecord - testing subfield deletion """

    def setUp(self):
        """Initialize stuff"""
        xml_example_record = """
        <record>
        <controlfield tag="001">33</controlfield>
        <datafield tag="041" ind1=" " ind2=" ">
        <subfield code="a">eng</subfield>
        </datafield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="a">Doe2, John</subfield>
        <subfield code="b">editor</subfield>
        <subfield code="z">Skal vi danse?</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2="1">
        <subfield code="a">On the foo and bar1</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2="2">
        <subfield code="a">On the foo and bar2</subfield>
        </datafield>
        </record>
        """
        self.rec = bibrecord.create_record(xml_example_record, 1, 1)[0]

    def test_delete_subfield_from(self):
        """bibrecord - delete subfield from position"""
        bibrecord.record_delete_subfield_from(self.rec, "100", 2,
            field_position_global=3)
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "100", " ", " ", "z"),
                         [])
        bibrecord.record_delete_subfield_from(self.rec, "100", 0,
            field_position_global=3)
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "100", " ", " ", "%"),
                         ['editor'])
        bibrecord.record_delete_subfield_from(self.rec, "100", 0,
            field_position_global=3)
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "100", " ", " ", "%"),
                         [])
        # Some crash tests
        self.assertRaises(bibrecord.InvenioBibRecordFieldError,
            bibrecord.record_delete_subfield_from, self.rec, "187", 0,
            field_position_global=1)
        self.assertRaises(bibrecord.InvenioBibRecordFieldError,
            bibrecord.record_delete_subfield_from, self.rec, "245", 0,
            field_position_global=999)
        self.assertRaises(bibrecord.InvenioBibRecordFieldError,
            bibrecord.record_delete_subfield_from, self.rec, "245", 999,
            field_position_global=4)


class BibRecordDeleteSubfieldTest(unittest.TestCase):
    """ bibrecord - testing subfield deletion """

    def setUp(self):
        """Initialize stuff"""
        self.xml_example_record = """
        <record>
        <controlfield tag="001">33</controlfield>
        <datafield tag="041" ind1=" " ind2=" ">
            <subfield code="a">eng</subfield>
        </datafield>
        <datafield tag="100" ind1=" " ind2=" ">
            <subfield code="a">Doe2, John</subfield>
            <subfield code="b">editor</subfield>
            <subfield code="z">Skal vi danse?</subfield>
            <subfield code="a">Doe3, Zbigniew</subfield>
            <subfield code="d">Doe4, Joachim</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2="1">
            <subfield code="a">On the foo and bar1</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2="2">
            <subfield code="a">On the foo and bar2</subfield>
        </datafield>
        <datafield tag="246" ind1="1" ind2="2">
            <subfield code="c">On the foo and bar1</subfield>
        </datafield>
        <datafield tag="246" ind1="1" ind2="2">
            <subfield code="c">On the foo and bar2</subfield>
        </datafield>
        </record>
        """

    def test_simple_removals(self):
        """ bibrecord - delete subfield by its code"""
        # testing a simple removals where all the fields are removed
        rec = bibrecord.create_record(self.xml_example_record, 1, 1)[0]

        bibrecord.record_delete_subfield(rec, "041", "b") # nothing should change
        self.assertEqual(rec["041"][0][0], [("a", "eng")])
        bibrecord.record_delete_subfield(rec, "041", "a")
        self.assertEqual(rec["041"][0][0], [])

    def test_indices_important(self):
        """ bibrecord - delete subfield where indices are important"""
        rec = bibrecord.create_record(self.xml_example_record, 1, 1)[0]
        bibrecord.record_delete_subfield(rec, "245", "a", " ", "1")
        self.assertEqual(rec["245"][0][0], [])
        self.assertEqual(rec["245"][1][0], [("a", "On the foo and bar2")])
        bibrecord.record_delete_subfield(rec, "245", "a", " ", "2")
        self.assertEqual(rec["245"][1][0], [])

    def test_remove_some(self):
        """ bibrecord - delete subfield when some should be preserved and some removed"""
        rec = bibrecord.create_record(self.xml_example_record, 1, 1)[0]
        bibrecord.record_delete_subfield(rec, "100", "a", " ", " ")
        self.assertEqual(rec["100"][0][0], [("b", "editor"), ("z", "Skal vi danse?"), ("d", "Doe4, Joachim")])

    def test_more_fields(self):
        """ bibrecord - delete subfield where more fits criteria"""
        rec = bibrecord.create_record(self.xml_example_record, 1, 1)[0]
        bibrecord.record_delete_subfield(rec, "246", "c", "1", "2")
        self.assertEqual(rec["246"][1][0], [])
        self.assertEqual(rec["246"][0][0], [])

    def test_nonexisting_removals(self):
        """ bibrecord - delete subfield that does not exist """
        rec = bibrecord.create_record(self.xml_example_record, 1, 1)[0]
        # further preparation
        bibrecord.record_delete_subfield(rec, "100", "a", " ", " ")
        self.assertEqual(rec["100"][0][0], [("b", "editor"), ("z", "Skal vi danse?"), ("d", "Doe4, Joachim")])
        #the real tests begin
        #   1) removing the subfield from an empty list of subfields
        bibrecord.record_delete_subfield(rec, "246", "c", "1", "2")
        self.assertEqual(rec["246"][1][0], [])
        self.assertEqual(rec["246"][0][0], [])
        bibrecord.record_delete_subfield(rec, "246", "8", "1", "2")
        self.assertEqual(rec["246"][1][0], [])
        self.assertEqual(rec["246"][0][0], [])
        #   2) removing a subfield from a field that has some subfields but none has an appropriate code
        bibrecord.record_delete_subfield(rec, "100", "a", " ", " ")
        self.assertEqual(rec["100"][0][0], [("b", "editor"), ("z", "Skal vi danse?"), ("d", "Doe4, Joachim")])
        bibrecord.record_delete_subfield(rec, "100", "e", " ", " ")
        self.assertEqual(rec["100"][0][0], [("b", "editor"), ("z", "Skal vi danse?"), ("d", "Doe4, Joachim")])


class BibRecordMoveSubfieldTest(unittest.TestCase):
    """ bibrecord - testing subfield moving """

    def setUp(self):
        """Initialize stuff"""
        xml_example_record = """
        <record>
        <controlfield tag="001">33</controlfield>
        <datafield tag="041" ind1=" " ind2=" ">
        <subfield code="a">eng</subfield>
        </datafield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="a">Doe2, John</subfield>
        <subfield code="b">editor</subfield>
        <subfield code="c">fisk</subfield>
        <subfield code="d">eple</subfield>
        <subfield code="e">hammer</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2="1">
        <subfield code="a">On the foo and bar1</subfield>
        </datafield>
        </record>
        """
        self.rec = bibrecord.create_record(xml_example_record, 1, 1)[0]

    def test_move_subfield(self):
        """bibrecord - move subfields"""
        bibrecord.record_move_subfield(self.rec, "100", 2, 4,
            field_position_global=3)
        bibrecord.record_move_subfield(self.rec, "100", 1, 0,
            field_position_global=3)
        bibrecord.record_move_subfield(self.rec, "100", 2, 999,
            field_position_global=3)
        self.assertEqual(bibrecord.record_get_field_values(self.rec, "100", " ", " ", "%"),
                         ['editor', 'Doe2, John', 'hammer', 'fisk', 'eple'])
        # Some crash tests
        self.assertRaises(bibrecord.InvenioBibRecordFieldError,
            bibrecord.record_move_subfield, self.rec, "187", 0, 1,
            field_position_global=3)
        self.assertRaises(bibrecord.InvenioBibRecordFieldError,
            bibrecord.record_move_subfield, self.rec, "100", 1, 0,
            field_position_global=999)
        self.assertRaises(bibrecord.InvenioBibRecordFieldError,
            bibrecord.record_move_subfield, self.rec, "100", 999, 0,
            field_position_global=3)


class BibRecordCompareSubfieldTest(unittest.TestCase):
    """ bibrecord -  """

    def setUp(self):
        """Initialize stuff"""
        xml_example_record = """
        <record>
        <controlfield tag="001">33</controlfield>
        <datafield tag="041" ind1=" " ind2=" ">
        <subfield code="a">eng</subfield>
        </datafield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="a">Doe2, John</subfield>
        <subfield code="b">editor</subfield>
        <subfield code="c">fisk</subfield>
        <subfield code="d">eple</subfield>
        <subfield code="e">hammer</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2="1">
        <subfield code="a">On the foo and bar1</subfield>
        </datafield>
        </record>
        """
        self.rec = bibrecord.create_record(xml_example_record, 1, 1)[0]
        # For simplicity, create an alias of the function
        self._func = bibrecord.record_match_subfields

    def test_check_subfield_exists(self):
        self.assertEqual(self._func(self.rec, '100', sub_key='a'), 3)
        self.assertEqual(self._func(self.rec, '100', sub_key='e'), 3)
        self.assertFalse(self._func(self.rec, '245', sub_key='a'))
        self.assertEqual(self._func(self.rec, '245', ind2='1', sub_key='a'), 4)
        self.assertFalse(self._func(self.rec, '999', sub_key='x'))
        self.assertFalse(self._func(self.rec, '100', sub_key='x'))

    def test_check_subfield_values(self):
        self.assertEqual(self._func(self.rec, '100', sub_key='b',
            sub_value='editor'), 3)
        self.assertEqual(self._func(self.rec, '245', ind2='1', sub_key='a',
            sub_value='On the foo and bar1'), 4)
        self.assertEqual(self._func(self.rec, '100', sub_key='e',
            sub_value='ponies suck'), False)
        self.assertEqual(self._func(self.rec, '100', sub_key='c',
            sub_value='FISK'), False)
        self.assertEqual(self._func(self.rec, '100', sub_key='c',
            sub_value='FISK', case_sensitive=False), 3)

    def test_compare_subfields(self):
        self.assertEqual(self._func(self.rec, '100', sub_key='c',
            sub_value='fisk', sub_key2='d', sub_value2='eple'), 3)
        self.assertFalse(self._func(self.rec, '100', sub_key='c',
            sub_value='fisk', sub_key2='d', sub_value2='tom'))
        self.assertEqual(self._func(self.rec, '100', sub_key='c',
            sub_value='fiSk', sub_key2='d', sub_value2='Eple',
            case_sensitive=False), 3)

    def test_error_conditions(self):
        self.assertRaises(TypeError,
            self._func, self.rec, '100')
        self.assertRaises(TypeError,
            self._func, self.rec, '100', sub_key='a',
                sub_value='fiSk', sub_key2='d')


class BibRecordSpecialTagParsingTest(unittest.TestCase):
    """ bibrecord - parsing special tags (FMT, FFT)"""

    def setUp(self):
        """setting up example records"""
        self.xml_example_record_with_fmt = """
        <record>
         <controlfield tag="001">33</controlfield>
         <datafield tag="041" ind1=" " ind2=" ">
          <subfield code="a">eng</subfield>
         </datafield>
         <datafield tag="FMT" ind1=" " ind2=" ">
          <subfield code="f">HB</subfield>
          <subfield code="g">Let us see if this gets inserted well.</subfield>
         </datafield>
        </record>
        """
        self.xml_example_record_with_fft = """
        <record>
         <controlfield tag="001">33</controlfield>
         <datafield tag="041" ind1=" " ind2=" ">
          <subfield code="a">eng</subfield>
         </datafield>
         <datafield tag="FFT" ind1=" " ind2=" ">
          <subfield code="a">file:///foo.pdf</subfield>
          <subfield code="a">http://bar.com/baz.ps.gz</subfield>
         </datafield>
        </record>
        """
        self.xml_example_record_with_xyz = """
        <record>
         <controlfield tag="001">33</controlfield>
         <datafield tag="041" ind1=" " ind2=" ">
          <subfield code="a">eng</subfield>
         </datafield>
         <datafield tag="XYZ" ind1=" " ind2=" ">
          <subfield code="f">HB</subfield>
          <subfield code="g">Let us see if this gets inserted well.</subfield>
         </datafield>
        </record>
        """

    def test_parsing_file_containing_fmt_special_tag_with_correcting(self):
        """bibrecord - parsing special FMT tag, correcting on"""
        rec = bibrecord.create_record(self.xml_example_record_with_fmt, 1, 1)[0]
        self.assertEqual(rec,
                         {u'001': [([], " ", " ", '33', 1)],
                          'FMT': [([('f', 'HB'), ('g', 'Let us see if this gets inserted well.')], " ", " ", "", 3)],
                          '041': [([('a', 'eng')], " ", " ", "", 2)]})
        self.assertEqual(bibrecord.record_get_field_values(rec, "041", " ", " ", "a"),
                         ['eng'])
        self.assertEqual(bibrecord.record_get_field_values(rec, "FMT", " ", " ", "f"),
                         ['HB'])
        self.assertEqual(bibrecord.record_get_field_values(rec, "FMT", " ", " ", "g"),
                         ['Let us see if this gets inserted well.'])

    def test_parsing_file_containing_fmt_special_tag_without_correcting(self):
        """bibrecord - parsing special FMT tag, correcting off"""
        rec = bibrecord.create_record(self.xml_example_record_with_fmt, 1, 0)[0]
        self.assertEqual(rec,
                         {u'001': [([], " ", " ", '33', 1)],
                          'FMT': [([('f', 'HB'), ('g', 'Let us see if this gets inserted well.')], " ", " ", "", 3)],
                          '041': [([('a', 'eng')], " ", " ", "", 2)]})
        self.assertEqual(bibrecord.record_get_field_values(rec, "041", " ", " ", "a"),
                         ['eng'])
        self.assertEqual(bibrecord.record_get_field_values(rec, "FMT", " ", " ", "f"),
                         ['HB'])
        self.assertEqual(bibrecord.record_get_field_values(rec, "FMT", " ", " ", "g"),
                         ['Let us see if this gets inserted well.'])

    def test_parsing_file_containing_fft_special_tag_with_correcting(self):
        """bibrecord - parsing special FFT tag, correcting on"""
        rec = bibrecord.create_record(self.xml_example_record_with_fft, 1, 1)[0]
        self.assertEqual(rec,
                         {u'001': [([], " ", " ", '33', 1)],
                          'FFT': [([('a', 'file:///foo.pdf'), ('a', 'http://bar.com/baz.ps.gz')], " ", " ", "", 3)],
                          '041': [([('a', 'eng')], " ", " ", "", 2)]})
        self.assertEqual(bibrecord.record_get_field_values(rec, "041", " ", " ", "a"),
                         ['eng'])
        self.assertEqual(bibrecord.record_get_field_values(rec, "FFT", " ", " ", "a"),
                         ['file:///foo.pdf', 'http://bar.com/baz.ps.gz'])

    def test_parsing_file_containing_fft_special_tag_without_correcting(self):
        """bibrecord - parsing special FFT tag, correcting off"""
        rec = bibrecord.create_record(self.xml_example_record_with_fft, 1, 0)[0]
        self.assertEqual(rec,
                         {u'001': [([], " ", " ", '33', 1)],
                          'FFT': [([('a', 'file:///foo.pdf'), ('a', 'http://bar.com/baz.ps.gz')], " ", " ", "", 3)],
                          '041': [([('a', 'eng')], " ", " ", "", 2)]})
        self.assertEqual(bibrecord.record_get_field_values(rec, "041", " ", " ", "a"),
                         ['eng'])
        self.assertEqual(bibrecord.record_get_field_values(rec, "FFT", " ", " ", "a"),
                         ['file:///foo.pdf', 'http://bar.com/baz.ps.gz'])

    def test_parsing_file_containing_xyz_special_tag_with_correcting(self):
        """bibrecord - parsing unrecognized special XYZ tag, correcting on"""
        # XYZ should not get accepted when correcting is on; should get changed to 000
        rec = bibrecord.create_record(self.xml_example_record_with_xyz, 1, 1)[0]
        self.assertEqual(rec,
                         {u'001': [([], " ", " ", '33', 1)],
                          '000': [([('f', 'HB'), ('g', 'Let us see if this gets inserted well.')], " ", " ", "", 3)],
                          '041': [([('a', 'eng')], " ", " ", "", 2)]})
        self.assertEqual(bibrecord.record_get_field_values(rec, "041", " ", " ", "a"),
                         ['eng'])
        self.assertEqual(bibrecord.record_get_field_values(rec, "XYZ", " ", " ", "f"),
                         [])
        self.assertEqual(bibrecord.record_get_field_values(rec, "XYZ", " ", " ", "g"),
                         [])
        self.assertEqual(bibrecord.record_get_field_values(rec, "000", " ", " ", "f"),
                         ['HB'])
        self.assertEqual(bibrecord.record_get_field_values(rec, "000", " ", " ", "g"),
                         ['Let us see if this gets inserted well.'])

    def test_parsing_file_containing_xyz_special_tag_without_correcting(self):
        """bibrecord - parsing unrecognized special XYZ tag, correcting off"""
        # XYZ should get accepted without correcting
        rec = bibrecord.create_record(self.xml_example_record_with_xyz, 1, 0)[0]
        self.assertEqual(rec,
                         {u'001': [([], " ", " ", '33', 1)],
                          'XYZ': [([('f', 'HB'), ('g', 'Let us see if this gets inserted well.')], " ", " ", "", 3)],
                          '041': [([('a', 'eng')], " ", " ", "", 2)]})
        self.assertEqual(bibrecord.record_get_field_values(rec, "041", " ", " ", "a"),
                         ['eng'])
        self.assertEqual(bibrecord.record_get_field_values(rec, "XYZ", " ", " ", "f"),
                         ['HB'])
        self.assertEqual(bibrecord.record_get_field_values(rec, "XYZ", " ", " ", "g"),
                         ['Let us see if this gets inserted well.'])


class BibRecordPrintingTest(unittest.TestCase):
    """ bibrecord - testing for printing record """

    def setUp(self):
        """Initialize stuff"""
        self.xml_example_record = """
        <record>
        <controlfield tag="001">81</controlfield>
        <datafield tag="037" ind1=" " ind2=" ">
        <subfield code="a">TEST-ARTICLE-2006-001</subfield>
        </datafield>
        <datafield tag="037" ind1=" " ind2=" ">
        <subfield code="a">ARTICLE-2006-001</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2=" ">
        <subfield code="a">Test ti</subfield>
        </datafield>
        </record>"""

        self.xml_example_record_short = """
        <record>
        <controlfield tag="001">81</controlfield>
        <datafield tag="037" ind1=" " ind2=" ">
        <subfield code="a">TEST-ARTICLE-2006-001</subfield>
        </datafield>
        <datafield tag="037" ind1=" " ind2=" ">
        <subfield code="a">ARTICLE-2006-001</subfield>
        </datafield>
        </record>"""

        self.xml_example_multi_records = """
        <record>
        <controlfield tag="001">81</controlfield>
        <datafield tag="037" ind1=" " ind2=" ">
        <subfield code="a">TEST-ARTICLE-2006-001</subfield>
        </datafield>
        <datafield tag="037" ind1=" " ind2=" ">
        <subfield code="a">ARTICLE-2006-001</subfield>
        </datafield>
        <datafield tag="245" ind1=" " ind2=" ">
        <subfield code="a">Test ti</subfield>
        </datafield>
        </record>
        <record>
        <controlfield tag="001">82</controlfield>
        <datafield tag="100" ind1=" " ind2=" ">
        <subfield code="a">Author, t</subfield>
        </datafield>
        </record>"""

        self.xml_example_multi_records_short = """
        <record>
        <controlfield tag="001">81</controlfield>
        <datafield tag="037" ind1=" " ind2=" ">
        <subfield code="a">TEST-ARTICLE-2006-001</subfield>
        </datafield>
        <datafield tag="037" ind1=" " ind2=" ">
        <subfield code="a">ARTICLE-2006-001</subfield>
        </datafield>
        </record>
        <record>
        <controlfield tag="001">82</controlfield>
        </record>"""

    def test_record_xml_output(self):
        """bibrecord - xml output"""
        rec = bibrecord.create_record(self.xml_example_record, 1, 1)[0]
        rec_short = bibrecord.create_record(self.xml_example_record_short, 1, 1)[0]
        self.assertEqual(bibrecord.create_record(bibrecord.record_xml_output(rec, tags=[]), 1, 1)[0], rec)
        self.assertEqual(bibrecord.create_record(bibrecord.record_xml_output(rec, tags=["001", "037"]), 1, 1)[0], rec_short)
        self.assertEqual(bibrecord.create_record(bibrecord.record_xml_output(rec, tags=["037"]), 1, 1)[0], rec_short)

class BibRecordCreateFieldTest(unittest.TestCase):
    """ bibrecord - testing for creating field """

    def test_create_valid_field(self):
        """bibrecord - create and check a valid field"""
        bibrecord.create_field()
        bibrecord.create_field([('a', 'testa'), ('b', 'testb')], '2', 'n',
            'controlfield', 15)

    def test_invalid_field_raises_exception(self):
        """bibrecord - exception raised when creating an invalid field"""
        # Invalid subfields.
        self.assertRaises(bibrecord.InvenioBibRecordFieldError,
            bibrecord.create_field, 'subfields', '1', '2', 'controlfield', 10)
        self.assertRaises(bibrecord.InvenioBibRecordFieldError,
            bibrecord.create_field, ('1', 'value'), '1', '2', 'controlfield', 10)
        self.assertRaises(bibrecord.InvenioBibRecordFieldError,
            bibrecord.create_field, [('value')], '1', '2', 'controlfield', 10)
        self.assertRaises(bibrecord.InvenioBibRecordFieldError,
            bibrecord.create_field, [('1', 'value', '2')], '1', '2', 'controlfield', 10)
        # Invalid indicators.
        self.assertRaises(bibrecord.InvenioBibRecordFieldError,
            bibrecord.create_field, [], 1, '2', 'controlfield', 10)
        self.assertRaises(bibrecord.InvenioBibRecordFieldError,
            bibrecord.create_field, [], '1', 2, 'controlfield', 10)
        # Invalid controlfield value
        self.assertRaises(bibrecord.InvenioBibRecordFieldError,
            bibrecord.create_field, [], '1', '2', 13, 10)
        # Invalid global position
        self.assertRaises(bibrecord.InvenioBibRecordFieldError,
            bibrecord.create_field, [], '1', '2', 'controlfield', 'position')

    def test_compare_fields(self):
        """bibrecord - compare fields"""
        # Identical
        field0 = ([('a', 'test')], '1', '2', '', 0)
        field1 = ([('a', 'test')], '1', '2', '', 3)
        self.assertEqual(True,
            bibrecord._compare_fields(field0, field1, strict=True))
        self.assertEqual(True,
            bibrecord._compare_fields(field0, field1, strict=False))

        # Order of the subfields changed.
        field0 = ([('a', 'testa'), ('b', 'testb')], '1', '2', '', 0)
        field1 = ([('b', 'testb'), ('a', 'testa')], '1', '2', '', 3)
        self.assertEqual(False,
            bibrecord._compare_fields(field0, field1, strict=True))
        self.assertEqual(True,
            bibrecord._compare_fields(field0, field1, strict=False))

        # Different
        field0 = ([], '3', '2', '', 0)
        field1 = ([], '1', '2', '', 3)
        self.assertEqual(False,
            bibrecord._compare_fields(field0, field1, strict=True))
        self.assertEqual(False,
            bibrecord._compare_fields(field0, field1, strict=False))

class BibRecordFindFieldTest(unittest.TestCase):
    """ bibrecord - testing for finding field """

    def setUp(self):
        """Initialize stuff"""
        xml = """
        <record>
        <controlfield tag="001">81</controlfield>
        <datafield tag="037" ind1=" " ind2=" ">
        <subfield code="a">TEST-ARTICLE-2006-001</subfield>
        <subfield code="b">ARTICLE-2007-001</subfield>
        </datafield>
        </record>
        """
        self.rec = bibrecord.create_record(xml)[0]
        self.field0 = self.rec['001'][0]
        self.field1 = self.rec['037'][0]
        self.field2 = (
            [self.field1[0][1], self.field1[0][0]],
            self.field1[1],
            self.field1[2],
            self.field1[3],
            self.field1[4],
            )

    def test_finding_field_strict(self):
        """bibrecord - test finding field strict"""
        self.assertEqual((1, 0),
            bibrecord.record_find_field(self.rec, '001', self.field0,
            strict=True))
        self.assertEqual((2, 0),
            bibrecord.record_find_field(self.rec, '037', self.field1,
            strict=True))
        self.assertEqual((None, None),
            bibrecord.record_find_field(self.rec, '037', self.field2,
            strict=True))

    def test_finding_field_loose(self):
        """bibrecord - test finding field loose"""
        self.assertEqual((1, 0),
            bibrecord.record_find_field(self.rec, '001', self.field0,
            strict=False))
        self.assertEqual((2, 0),
            bibrecord.record_find_field(self.rec, '037', self.field1,
            strict=False))
        self.assertEqual((2, 0),
            bibrecord.record_find_field(self.rec, '037', self.field2,
            strict=False))

class BibRecordSingletonTest(unittest.TestCase):
    """ bibrecord - testing singleton removal """

    def setUp(self):
        """Initialize stuff"""
        self.xml = """<collection>
                       <record>
                         <controlfield tag="001">33</controlfield>
                         <controlfield tag="002" />
                         <datafield tag="99" ind1=" " ind2=" "/>
                         <datafield tag="100" ind1=" " ind2=" ">
                           <subfield code="a" />
                         </datafield>
                         <datafield tag="100" ind1=" " ind2=" ">
                           <subfield code="a">Some value</subfield>
                         </datafield>
                         <tagname />
                       </record>
                       <record />
                      <collection>"""
        self.rec_expected = {
            '001': [([], ' ', ' ', '33', 1)],
            '100': [([('a', 'Some value')], ' ', ' ', '', 2)],
            }


    def test_singleton_removal_lxml(self):
        """bibrecord - enforcing singleton removal with lxml"""
        rec = bibrecord.create_records(self.xml, verbose=1,
                                       correct=1, parser='lxml',
                                       keep_singletons=False)[0][0]
        self.assertEqual(rec, self.rec_expected)

class BibRecordNumCharRefTest(unittest.TestCase):
    """ bibrecord - testing numerical character reference expansion"""

    def setUp(self):
        """Initialize stuff"""
        self.xml = """<?xml version="1.0" encoding="UTF-8"?>
                      <record>
                        <controlfield tag="001">33</controlfield>
                        <datafield tag="123" ind1=" " ind2=" ">
                          <subfield code="a">Σ &amp; &#931;</subfield>
                          <subfield code="a">use &amp;amp; in XML</subfield>
                        </datafield>
                      </record>"""
        self.rec_expected = {
            '001': [([], ' ', ' ', '33', 1)],
            '123': [([('a', '\xce\xa3 & \xce\xa3'), ('a', 'use &amp; in XML'),], ' ', ' ', '', 2)],
            }

    def test_numcharref_expansion_lxml(self):
        """bibrecord - numcharref expansion with lxml"""
        rec = bibrecord.create_records(self.xml, verbose=1,
                                       correct=1, parser='lxml')[0][0]
        self.assertEqual(rec, self.rec_expected)


class BibRecordExtractIdentifiersTest(unittest.TestCase):
    """ bibrecord - testing for getting identifiers from record """

    def setUp(self):
        """Initialize stuff"""
        from harvestingkit.bibrecord import CFG_BIBUPLOAD_EXTERNAL_OAIID_TAG
        xml_example_record = """
        <record>
        <controlfield tag="001">1</controlfield>
        <datafield tag="100" ind1="C" ind2="5">
        <subfield code="a">val1</subfield>
        </datafield>
        <datafield tag="024" ind1="7" ind2=" ">
        <subfield code="2">doi</subfield>
        <subfield code="a">5555/TEST1</subfield>
        </datafield>
        <datafield tag="024" ind1="7" ind2=" ">
        <subfield code="2">DOI</subfield>
        <subfield code="a">5555/TEST2</subfield>
        </datafield>
        <datafield tag="024" ind1="7" ind2=" ">
        <subfield code="2">nondoi</subfield>
        <subfield code="a">5555/TEST3</subfield>
        </datafield>
        <datafield tag="024" ind1="8" ind2=" ">
        <subfield code="2">doi</subfield>
        <subfield code="a">5555/TEST4</subfield>
        </datafield>
        <datafield tag="%(oai_tag)s" ind1="%(oai_ind1)s" ind2="%(oai_ind2)s">
        <subfield code="%(oai_subcode)s">oai:atlantis:1</subfield>
        </datafield>
        </record>
        """ % {'oai_tag': CFG_BIBUPLOAD_EXTERNAL_OAIID_TAG[0:3],
               'oai_ind1': CFG_BIBUPLOAD_EXTERNAL_OAIID_TAG[3],
               'oai_ind2': CFG_BIBUPLOAD_EXTERNAL_OAIID_TAG[4],
               'oai_subcode': CFG_BIBUPLOAD_EXTERNAL_OAIID_TAG[5],
               }
        self.rec = bibrecord.create_record(xml_example_record, 1, 1)[0]

    def test_extract_doi(self):
        """bibrecord - getting DOI identifier(s) from record"""
        self.assertEqual(bibrecord.record_extract_dois(self.rec),
                         ['5555/TEST1', '5555/TEST2'])


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#
# This file is part of Harvesting Kit.
# Copyright (C) 2014, 2015 CERN.
#
# Harvesting Kit is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Harvesting Kit is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Harvesting Kit; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.


"""Unit tests for APS."""

import unittest

from harvestingkit.aps_package import ApsPackage
from xml.dom.minidom import parse
from os.path import (join,
                     dirname)
from harvestingkit.tests import (__file__ as folder,
                                 aps_test_record,
                                 aps_output,
                                 journal_mappings)


class APSPackageTests(unittest.TestCase):

    """Test that metadata are exported correctly."""

    def setUp(self):
        """Setup sample parsing used in tests."""
        self.aps = ApsPackage(journal_mappings)
        self.aps.document = parse(join(dirname(folder), aps_test_record))

    def test_journal(self):
        """Test that journal name is extracted correctly."""
        self.assertEqual(self.aps._get_journal(), 'Phys. Rev. D')

    def test_abstract(self):
        """Test that abstract is extracted correctly."""
        abstract = '<p>In conformally flat background geometries the' \
                   ' long-wavelength gravitons can be described in the ' \
                   'fluid approximation and they induce scalar fluctuations ' \
                   'both during inflation and in the subsequent ' \
                   'radiation-dominated epoch. While this effect is minute ' \
                   'and suppressed for a de Sitter stage of expansion, the ' \
                   'fluctuations of the energy-momentum pseudotensor of the ' \
                   'graviton fluid lead to curvature perturbations that ' \
                   'increase with time all along the post-inflationary evolution.' \
                   ' An explicit calculation of these effects is presented for' \
                   ' a standard thermal history and it is shown that the growth' \
                   ' of the curvature perturbations caused by the long-wavelength ' \
                   'modes is approximately compensated by the slope of the power ' \
                   'spectra of the energy density, pressure and anisotropic ' \
                   'stress of the relic gravitons.</p>'
        self.assertEqual(self.aps._get_abstract(), abstract)

    def test_title(self):
        """Check that title is correct."""
        title = 'Scalar modes of the relic gravitons', '', []
        self.assertEqual(self.aps._get_title(), title)

    def test_doi(self):
        """Check that DOI is correct."""
        self.assertEqual(self.aps._get_doi(), '10.1103/PhysRevD.91.023521')

    def test_authors(self):
        """Check that authors are correct."""
        authors = [('Giovannini, Massimo', [u'a1'], [u'n1'])]
        self.assertEqual(self.aps._get_authors(), authors)

    def test_affiliations(self):
        """Check that affiliations are correct."""
        affiliations = {
            u'a1': 'Department of Physics, Theory Division, CERN , 1211 Geneva 23, Switzerland INFN, Section of Milan-Bicocca, 20126 Milan, Italy'
        }
        self.assertEqual(self.aps._get_affiliations(), affiliations)

    def test_author_emails(self):
        """Check email from author."""
        emails = {u'n1': ['massimo.giovannini@cern.ch']}
        self.assertEqual(self.aps._get_author_emails(), emails)

    def test_copyright(self):
        """Check that Copyright is extracted."""
        self.assertEqual(self.aps._get_copyright(), ('authors', '2015', 'Published by the American Physical Society'))

    def test_date(self):
        """Check published date."""
        self.assertEqual(self.aps._get_date(), '2015-01-29')

    def test_publisher(self):
        """Check correct publisher."""
        self.assertEqual(self.aps._get_publisher(), 'American Physical Society')

    def test_publication_information(self):
        """Check extracted pubinfo."""
        publication_information = ('Phys.Rev.',
                                   'D91',
                                   '2',
                                   u'2015',
                                   u'2015-01-29',
                                   u'10.1103/PhysRevD.91.023521',
                                   '023521',
                                   '',
                                   '')
        self.assertEqual(self.aps._get_publication_information(), publication_information)

    def test_pagecount(self):
        """Check pagecount."""
        self.assertEqual(self.aps._get_page_count(), '15')

    def test_pacscodes(self):
        """Check that PACS are extracted."""
        self.assertEqual(self.aps._get_pacscodes(), ['98.80.Cq', '04.30.-w', '04.62.+v', '98.70.Vc'])

    def test_subject(self):
        """Check subject."""
        self.assertEqual(self.aps._get_subject(), 'Cosmology')

    def test_license(self):
        """Check license."""
        self.assertEqual(
             self.aps._get_license(),
             ('Creative Commons Attribution 3.0 License',
              'creative-commons',
              'http://creativecommons.org/licenses/by/3.0/')
        )

    def test_keywords(self):
        """Check keywords."""
        self.assertEqual(self.aps._get_keywords(), [])

    def test_references(self):
        """Check references."""
        references = [
            (u'journal', '', [u'L.\u2009P. Grishchuk'], '', 'Zh.\xc3\x89ksp.Teor.Fiz.',
             '67', '825', '1974', '1', '', '', '', [], '', '', []),
            (u'journal', '', [u'L.\u2009P. Grishchuk'], '', 'Sov.Phys.JETP',
             '40', '409', '1975', '1', '', '', '', [], '', '', []),
            (u'journal', '10.1111/j.1749-6632.1977.tb37064.x', [u'L.\u2009P. Grishchuk'],
             '', 'Ann.N.Y.Acad.Sci.', '302', '439', '1977', '1', '', '', '', [], '', '', []),
            (u'journal', '10.1111/j.1749-6632.1977.tb37064.x', [u'L.\u2009P. Grishchuk'], '',
             'Ann.N.Y.Acad.Sci.', '302', '439', '1977', '1', '', '', '', [], '', '', []),
            (u'journal', '', [u'A.\u2009A. Starobinsky'], '', 'JETP Lett.',
             '30', '682', '1979', '2', '', '', '', [], '', '', []),
            (u'journal', '10.1016/0370-2693(82)90641-4', [u'V.\u2009A. Rubakov', u'M.\u2009V. Sazhin', u'A.\u2009V. Veryaskin'],
             '', 'Phys.Lett.', 'B115', '189', '1982', '2', '', '', '', [], '', '', []),
            (u'journal', '10.1016/0370-2693(83)91322-9', [u'R. Fabbri', u'M.\u2009D. Pollock'],
             '', 'Phys.Lett.', 'B125', '445', '1983', '3', '', '', '', [], '', '', []),
            (u'journal', '10.1016/0550-3213(84)90329-8', [u'L.\u2009F. Abbott', u'M.\u2009B. Wise'],
             '', 'Nucl.Phys.', '244', '541', '1984', '3', '', '', '', [], '', '', []),
            (u'journal', '10.1103/PhysRevD.43.2566', [u'L.\u2009P. Grishchuk', u'M. Solokhin'], '',
             'Phys.Rev.', 'D43', '2566', '1991', '4', '', '', '', [], '', '', []),
            (u'journal', '10.1103/PhysRevD.42.453', [u'V. Sahni'], '', 'Phys.Rev.',
             'D42', '453', '1990', '4', '', '', '', [], '', '', []),
            (u'journal', '10.1103/PhysRevD.58.083504', [u'M. Giovannini'], '', 'Phys.Rev.',
             'D58', '083504', '1998', '5', '', '', '', [], '', '', []),
            (u'journal', '10.1103/PhysRevD.60.123511', [u'M. Giovannini'], '', 'Phys.Rev.',
             'D60', '123511', '1999', '5', '', '', '', [], '', '', []),
            (u'journal', '10.1088/0264-9381/26/4/045004', [u'M. Giovannini'], '',
             'Classical Quantum Gravity', '26', '045004', '2009', '5', '', '', '', [], '', '', []),
            (u'journal', '10.1016/j.physletb.2009.09.018', [u'W. Zhao', u'D. Baskaran', u'P. Coles'],
             '', 'Phys.Lett.', 'B680', '411', '2009', '5', '', '', '', [], '', '', []),
            (u'journal', '10.1103/PhysRevD.80.042002', [u'M.\u2009S. Pshirkov', u'D. Baskaran'],
             '', 'Phys.Rev.', 'D80', '042002', '2009', '6', '', '', '', [], '', '', []),
            (u'journal', '10.1103/PhysRevD.81.083503', [u'T. Chiba', u'K. Kamada', u'M. Yamaguchi'],
             '', 'Phys.Rev.', 'D81', '083503', '2010', '6', '', '', '', [], '', '', []),
            (u'journal', '10.1103/PhysRevD.89.123513', [u'M.\u2009W. Hossain', u'R. Myrzakulov', u'M. Sami', u'E.\u2009N. Saridakis'],
             '', 'Phys.Rev.', 'D89', '123513', '2014', '6', '', '', '', [], '', '', []),
            (u'book', '', [u'C.\u2009W. Misner', u'K.\u2009S. Thorne', u'J.\u2009A. Wheeler'],
             '', 'Gravitation', '', '467', '1973', '7', '', 'Freeman', '', u'New York,', '', '', []),
            (u'book', '', [u'S. Weinberg'], '', 'Gravitation and Cosmology',
             '', '166', '1972', '8', '', 'Wiley', '', u'New York,', '', '', []),
            (u'book', '', [u'L.\u2009D. Landau', u'E.\u2009M. Lifshitz'], '',
             'The Classical Theory of Fields', '', '', '1971', '9', '', 'Pergamon Press', '', u'New York,', '', '', []),
            (u'journal', '10.1103/PhysRev.166.1263', [u'R. Isaacson'], '', 'Phys.Rev.', '166', '1263',
             '1968', '10', '', '', '', [], '', '', []),
            (u'journal', '10.1103/PhysRev.166.1272', [u'R. Isaacson'], '', 'Phys.Rev.', '166', '1272',
             '1968', '10', '', '', '', [], '', '', []),
            (u'journal', '10.1103/PhysRevD.56.3248', [u'L.\u2009R. Abramo', u'R. Brandenberger', u'V. Mukahanov'],
             '', 'Phys.Rev.', 'D56', '3248', '1997', '11', '', '', '', [], '', '', []),
            (u'journal', '10.1103/PhysRevD.60.064004', [u'L.\u2009R. Abramo'], '',
             'Phys Rev.', 'D60', '064004', '1999', '11', '', '', '', [], '', '', []),
            (u'journal', '10.1103/PhysRevD.61.024038', [u'S.\u2009V. Babak', u'L.\u2009P. Grishchuk'],
             '', 'Phys.Rev.', 'D61', '024038', '1999', '11', '', '', '', [], '', '', []),
            (u'journal', '10.1103/PhysRevD.73.083505', [u'M. Giovannini'], '',
             'Phys.Rev.', 'D73', '083505', '2006', '12', '', '', '', [], '', '', []),
            (u'journal', '10.1103/PhysRevD.85.104012', [u'D. Su', u'Y. Zhang'],
             '', 'Phys.Rev.', 'D85', '104012', '2012', '12', '', '', '', [], '', '', []),
            (u'journal', '10.1103/PhysRevD.16.1601', [u'L.\u2009H. Ford', u'L. Parker'],
             '', 'Phys.Rev.', 'D16', '1601', '1977', '13', '', '', '', [], '', '', []),
            (u'journal', '10.1103/PhysRevD.16.245', [u'L.\u2009H. Ford', u'L. Parker'],
             '', 'Phys.Rev.', 'D16', '245', '1977', '13', '', '', '', [], '', '', []),
            (u'journal', '10.1016/0375-9601(77)90880-5', [u'B.\u2009L. Hu', u'L. Parker'],
             '', 'Phys.Lett', 'A63', '217', '1977', '13', '', '', '', [], '', '', []),
            (u'journal', '10.1103/PhysRevD.22.1882', [u'J. Bardeen'], '',
             'Phys.Rev.', 'D22', '1882', '1980', '14', '', '', '', [], '', '', []),
            (u'journal', '10.1093/mnras/200.3.535', [u'G.\u2009V. Chibisov', u'V.\u2009F. Mukhanov'],
             '', 'Mon.Not.R.Astron.Soc.', '200', '535', '1982', '14', '', '', '', [], '', '', []),
            (u'journal', '10.1103/PhysRevD.28.679', [u'J. Bardeen', u'P. Steinhardt', u'M. Turner'],
             '', 'Phys.Rev.', 'D28', '679', '1983', '14', '', '', '', [], '', '', []),
            (u'journal', '10.1103/PhysRevD.30.265', [u'J.\u2009A. Frieman', u'M.\u2009S. Turner'],
             '', 'Phys.Rev.', 'D30', '265', '1984', '14', '', '', '', [], '', '', []),
            (u'journal', '10.1143/PTPS.78.1', [u'H. Kodama', u'M. Sasaki'], '',
             'Prog.Theor.Phys.Suppl.', '78', '1', '1984', '14', '', '', '', [], '', '', []),
            (u'journal', '10.1086/170206', [u'J-c. Hwang'], '',
             'Astrophys.J.', '375', '443', '1991', '14', '', '', '', [], '', '', []),
            (u'journal', '', [u'V.\u2009N. Lukash'], '',
             'Zh.Eksp.Teor.Fiz.', '79', '1601', '1980', '15', '', '', '', [], '', '', []),
            (u'journal', '', [u'V.\u2009N. Lukash'], '',
             'Sov.Phys.JETP', '52', '807', '1980', '15', '', '', '', [], '', '', []),
            (u'journal', '10.1134/S1063772907060017', [u'V. Strokov'],
             '', 'Astronomy Reports', '51', '431', '2007', '15', '', '', '', [], '', '', []),
            (u'journal', '10.1088/0067-0049/192/2/15', [u'B. Gold'],
             '', 'Astrophys.J.Suppl.Ser.', '192', '15', '2011', '16', '', '', '', [], '', '', []),
            (u'journal', '10.1088/0067-0049/192/2/16', [u'D. Larson'],
             '', 'Astrophys.J.Suppl.Ser.', '192', '16', '2011', '16', '', '', '', [], '', '', []),
            (u'journal', '10.1088/0067-0049/192/2/17', [u'C.\u2009L. Bennett'],
             '', 'Astrophys.J.Suppl.Ser.', '192', '17', '2011', '16', '', '', '', [], '', '', []),
            (u'journal', '10.1088/0067-0049/208/2/19', [u'G. Hinshaw'],
             '', 'Astrophys.J.Suppl.Ser.', '208', '19', '2013', '16', '', '', '', [], '', '', []),
            (u'journal', '10.1088/0067-0049/208/2/20', [u'C.\u2009L. Bennett'],
             '', 'Astrophys.J.Suppl.Ser.', '208', '20', '2013', '16', '', '', '', [], '', '', []),
            (u'journal', '10.1086/377226', [u'D.\u2009N. Spergel'],
             '', 'Astrophys.J.Suppl.Ser.', '148', '175', '2003', '17', '', '', '', [], '', '', []),
            (u'journal', '10.1086/513700', [u'D.\u2009N. Spergel'],
             '', 'Astrophys.J.Suppl.Ser.', '170', '377', '2007', '17', '', '', '', [], '', '', []),
            (u'journal', '10.1086/513699', [u'L. Page'], '',
             'Astrophys.J.Suppl.Ser.', '170', '335', '2007', '17', '', '', '', [], '', '', []),
            (u'journal', '10.1103/PhysRevLett.112.241101',
             [u'P.\u2009A.\u2009R. Ade'], 'BICEP2 Collaboration', 'Phys.Rev.Lett.', '112', '241101', '2014', '18', '', '', '', [], '', '', []),
            (u'journal', '10.1088/0004-637X/792/1/62', [u'P.\u2009A.\u2009R. Ade'],
             'BICEP2 Collaboration', 'Astrophys.J.', '792', '62', '2014', '18', '', '', '', [], '', '', []),
            (u'journal', '10.1103/PhysRevD.8.4231', [u'G.\u2009L. Murphy'],
             '', 'Phys.Rev.', 'D8', '4231', '1973', '19', '', '', '', [], '', '', []),
            (u'journal', '10.1016/0375-9601(77)90953-7', [u'G.\u2009L. Murphy'],
             '', 'Phys.Lett.', 'A62', '75', '1977', '19', '', '', '', [], '', '', []),
            (u'journal', '', [u'V.\u2009A. Belinskii', u'I.\u2009M. Khalatnikov'],
             '', 'Zh.Eksp.Teor.Fiz.', '69', '401', '1975', '20', '', '', '', [], '', '', []),
            (u'journal', '', [u'V.\u2009A. Belinskii', u'I.\u2009M. Khalatnikov'],
             '', 'Sov.Phys.JETP', '42', '205', '1976', '20', '', '', '', [], '', '', []),
            (u'journal', '', [u'V.\u2009A. Belinskii', u'I.\u2009M. Khalatnikov'],
             '', 'Zh.Eksp.Teor.Fiz.Pis.Red.', '21', '223', '1975', '20', '', '', '', [], '', '', []),
            (u'journal', '', [u'V.\u2009A. Belinskii', u'I.\u2009M. Khalatnikov'],
             '', 'JETP Lett.', '21', '99', '1975', '20', '', '', '', [], '', '', []),
            (u'journal', '10.1103/PhysRevD.72.043514', [u'S. Weinberg'],
             '', 'Phys.Rev.', 'D72', '043514', '2005', '21', '', '', '', [], '', '', []),
            (u'journal', '10.1103/PhysRevD.74.023508', [u'S. Weinberg'],
             '', 'Phys.Rev.', 'D74', '023508', '2006', '21', '', '', '', [], '', '', []),
            (u'journal', '10.1088/0305-4470/35/5/312', [u'A. Romeo', u'A.A. Saharian'],
             '', 'J.Phys.', 'A35', '1297', '2002', '22', '', '', '', [], '', '', []),
            (u'book', '', [u'A.A. Saharian'], '', 
             'The Generalized Abel-Plana Formula with Applications to Bessel Functions and Casimir Effect', 
             '', '', '2008', '22', '', 
             'Yerevan State University Publishing House', '', u'Yerevan,', '', '', []),
            (u'report', '', [u'A. Romeo', u'A.A. Saharian'], 
             '', '', '', '', '', '22', '', '', '', u'Report No ICTP/2007/082', '', '', [])
        ]
        for ref in self.aps.document.getElementsByTagName('ref'):
            for innerref in self.aps._get_reference(ref):
                self.assertTrue(innerref in references)
    def test_article_type(self):
        """Check extracted article type."""
        self.assertEqual(self.aps._get_article_type(), 'research-article')

    def test_get_record(self):
        """Check full record conversion."""
        source_file = join(dirname(folder), aps_test_record)
        marc_file = join(dirname(folder), aps_output)
        with open(marc_file) as marc:
            result = marc.read()
        xml = self.aps.get_record(source_file)
        self.assertEqual(xml.strip(), result.strip())

if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(APSPackageTests)
    unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-
#
# This file is part of Harvesting Kit.
# Copyright (C) 2015 CERN.
#
# Harvesting Kit is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Harvesting Kit is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Harvesting Kit; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Tests for inspire_cds_package."""

import os
import unittest
import pkg_resources


class TestConversions(unittest.TestCase):

    """Test simple functionality."""

    def setUp(self):
        """Load demo data."""
        self.inspire_demo_data_path_oai = pkg_resources.resource_filename(
            'harvestingkit.tests',
            os.path.join('data', 'sample_inspire_oai.xml')
        )
        self.inspire_demo_data_path = pkg_resources.resource_filename(
            'harvestingkit.tests',
            os.path.join('data', 'sample_inspire.xml')
        )

    def test_record_parsing(self):
        """Test parsing of sample file."""
        from harvestingkit.bibrecord import BibRecordPackage

        bibrecs = BibRecordPackage(self.inspire_demo_data_path_oai)
        bibrecs.parse()
        self.assertEqual(len(bibrecs.get_records()), 5)

    def test_record_config_load(self):
        """Test loading of kbs."""
        from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS
        from harvestingkit.inspire_cds_package.from_cds import CDS2Inspire

        conversion = Inspire2CDS({})
        self.assertTrue(conversion.kbs)
        self.assertEqual(
            conversion.get_config_item("Dutch", "languages"),
            "dut"
        )

        conversion = CDS2Inspire({})
        self.assertTrue(conversion.kbs)
        self.assertEqual(
            conversion.get_config_item("Inf. Sci.", "journals"),
            "Info.Sci."
        )

    def test_multiple_conversions(self):
        """Test conversion of multiple records."""
        from harvestingkit.bibrecord import BibRecordPackage
        from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS

        bibrecs = BibRecordPackage(self.inspire_demo_data_path_oai)
        bibrecs.parse()
        xml = Inspire2CDS.convert_all(bibrecs.get_records())
        self.assertEqual(xml.count("</record>"), 5)
        self.assertEqual(xml.count('<controlfield tag="003">SzGeCERN</controlfield>'), 5)

    def test_non_oai_conversion(self):
        """Test conversion of non-OAI-PMH input MARCXML."""
        from harvestingkit.bibrecord import BibRecordPackage
        from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS

        bibrecs = BibRecordPackage(self.inspire_demo_data_path)
        bibrecs.parse()
        xml = Inspire2CDS.convert_all(bibrecs.get_records())
        self.assertEqual(xml.count("</record>"), 3)
        self.assertEqual(xml.count('<controlfield tag="003">SzGeCERN</controlfield>'), 3)

    def test_single_conversion(self):
        """Test conversion of non-OAI-PMH input MARCXML."""
        from harvestingkit.bibrecord import BibRecordPackage
        from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS

        bibrecs = BibRecordPackage(self.inspire_demo_data_path)
        bibrecs.parse()
        conversion = Inspire2CDS(bibrecs.get_records()[0])
        xml = conversion.convert()
        self.assertEqual(xml.count("</record>"), 1)
        self.assertEqual(xml.count('<controlfield tag="003">SzGeCERN</controlfield>'), 1)

    def test_is_not_published(self):
        """Test if published is correct."""
        from harvestingkit.bibrecord import BibRecordPackage
        from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS

        bibrecs = BibRecordPackage(self.inspire_demo_data_path)
        bibrecs.parse()
        non_published_record = Inspire2CDS(bibrecs.get_records()[0])
        self.assertFalse(non_published_record.is_published())

    def test_is_published(self):
        """Test if published is correct."""
        from harvestingkit.bibrecord import BibRecordPackage
        from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS

        bibrecs = BibRecordPackage(self.inspire_demo_data_path_oai)
        bibrecs.parse()
        non_published_record = Inspire2CDS(bibrecs.get_records()[0])
        self.assertTrue(non_published_record.is_published())


class TestINSPIRE2CDS(unittest.TestCase):

    """Test converted record data."""

    def setUp(self):
        """Load demo data."""
        from harvestingkit.bibrecord import BibRecordPackage
        from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS

        self.inspire_demo_data_path = pkg_resources.resource_filename(
            'harvestingkit.tests',
            os.path.join('data', 'sample_inspire_oai.xml')
        )

        bibrecs = BibRecordPackage(self.inspire_demo_data_path)
        bibrecs.parse()
        self.parsed_record = bibrecs.get_records()[0]
        self.package = Inspire2CDS(self.parsed_record)
        self.recid = self.package.get_recid()
        self.converted_record = self.package.get_record()

    def test_double_conversion(self):
        """Test that calling things twice does not alter results."""
        self.assertEqual(self.converted_record,
                         self.package.get_record())

    def test_inspire_id(self):
        """Test for INSPIRE ID in 035."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertTrue(
            record_get_field_values(self.converted_record,
                                    tag="035",
                                    code="a",
                                    filter_subfield_code="a",
                                    filter_subfield_value=self.recid,
                                    filter_subfield_mode="e")
        )

    def test_cds_id(self):
        """Test for INSPIRE ID in 035."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertTrue(
            record_get_field_values(self.converted_record,
                                    tag="001")
        )

    def test_pubnote(self):
        """Check that the 773 field is good."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="773",
                                    code="v"),
            ["768"]
        )
        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="773",
                                    code="p"),
            ["Nucl. Instrum. Methods Phys. Res., A"]
        )

    def test_language(self):
        """Test for language."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="041",
                                    code="a"),
            ["eng"]
        )

    def test_page_number(self):
        """Test for page number."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="300",
                                    code="a"),
            ["8 p"]
        )

    def test_date(self):
        """Test for proper date in 260."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="260",
                                    code="c"),
            ["2014"]
        )

    def test_subject(self):
        """Test for subject in 65017."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertTrue(
            record_get_field_values(self.converted_record,
                                    tag="650", ind1="1", ind2="7",
                                    code="a",
                                    filter_subfield_code="a",
                                    filter_subfield_value="Detectors and Experimental Techniques",
                                    filter_subfield_mode="e")
        )

    def test_notex(self):
        """Make sure that no SPIRES/INSPIRE TeX code is there."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertFalse(
            record_get_field_values(self.converted_record,
                                    tag="035",
                                    code="9",
                                    filter_subfield_code="9",
                                    filter_subfield_value="INSPIRETeX",
                                    filter_subfield_mode="e")
        )

    def test_nodesy(self):
        """Make sure that no DESY fields are there."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertFalse(
            record_get_field_values(self.converted_record,
                                    tag="035",
                                    code="9",
                                    filter_subfield_code="9",
                                    filter_subfield_value="DESY",
                                    filter_subfield_mode="e")
        )

    def test_nocds(self):
        """Make sure that no CDS fields are there."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertFalse(
            record_get_field_values(self.converted_record,
                                    tag="035",
                                    code="9",
                                    filter_subfield_code="9",
                                    filter_subfield_value="CDS",
                                    filter_subfield_mode="e")
        )

    def test_nohiddenfield(self):
        """Make sure that no hidden INSPIRE field is there."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertFalse(
            record_get_field_values(self.converted_record,
                                    tag="595",
                                    code="%")
        )

    def test_noinspirenotes(self):
        """Make sure that no INSPIRE specific notes are there."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertFalse(
            record_get_field_values(self.converted_record,
                                    tag="500",
                                    code="a",
                                    filter_subfield_code="a",
                                    filter_subfield_value="*Brief entry*",
                                    filter_subfield_mode="s")
        )
        # Some notes should be there
        self.assertTrue(
            record_get_field_values(self.converted_record,
                                    tag="500",
                                    code="a")
        )

    def test_experiments(self):
        """Test for correct experiments values in 693."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="693",
                                    code="a"),
            ["CERN LHC"]
        )
        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="693",
                                    code="b"),
            ["T9", "T10"]
        )
        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="693",
                                    code="e"),
            ["CMS"]
        )

    def test_isbn(self):
        """Test for correct ISBN."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="020",
                                    code="a"),
            ["9785949900109"]
        )

    def test_duplicate_dois(self):
        """Test for correct removal of duplicate DOIs."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="024",
                                    ind1="7",
                                    code="a"),
            ["10.1016/j.nima.2014.09.061"]
        )

    def test_fft(self):
        """Test for existence of FFT on PDF URL."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertTrue(
            record_get_field_values(self.converted_record,
                                    tag="FFT",
                                    code="a")
        )
        # Some other URL should be there
        self.assertTrue(
            record_get_field_values(self.converted_record,
                                    tag="856",
                                    ind1="4",
                                    code="u")
        )

    def test_cern_tag(self):
        """Test for CERN tag in 690."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="690",
                                    ind1="C",
                                    code="a"),
            ["CERN"]
        )

    def test_oai_tag(self):
        """Test for OAI tag in 035."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="035",
                                    code="m"),
            ["marcxml"]
        )

    def test_oai_tag_024(self):
        """Test for OAI tag in 024."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="024",
                                    ind1="8",
                                    code="p"),
            ["CERN", "INSPIRE:HEP", "ForCDS"]
        )

    def test_no_journal_substring(self):
        """Test for correct journal conversion without substring matching."""
        from harvestingkit.bibrecord import record_add_field, record_get_field_values
        from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS
        rec = {}
        record_add_field(rec, tag='773', subfields=[('p', 'J. Phys.')])
        record_add_field(rec, tag='773', subfields=[('p', 'J.Phys.')])
        r = Inspire2CDS(rec)
        r.update_journals()
        self.assertNotEqual(record_get_field_values(rec, '773', code='p'),
            ['J. Phys.', 'Czechoslov. J. Phys.'])
        self.assertEqual(record_get_field_values(rec, '773', code='p'),
            ['J. Phys.', 'J. Phys.'])


class TestINSPIRE2CDSProceeding(unittest.TestCase):

    """Test converted record data."""

    def setUp(self):
        """Load demo data."""
        from harvestingkit.bibrecord import BibRecordPackage
        from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS

        self.inspire_demo_data_path = pkg_resources.resource_filename(
            'harvestingkit.tests',
            os.path.join('data', 'sample_inspire.xml')
        )

        bibrecs = BibRecordPackage(self.inspire_demo_data_path)
        bibrecs.parse()
        self.parsed_record = bibrecs.get_records()[0]
        self.package = Inspire2CDS(self.parsed_record)
        self.recid = self.package.get_recid()
        self.converted_record = self.package.get_record()

    def test_conference_tag(self):
        """Test for conference tag in 690C."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="690",
                                    ind1="C",
                                    code="a"),
            ["CONFERENCE", "CERN"]
        )

    def test_cnum(self):
        """Make sure that CNUM is okay."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="773",
                                    code="w"),
            ["C10-09-06.10"]
        )
        values = record_get_field_values(
            self.converted_record,
            tag="035",
            code="9"
        )
        self.assertTrue("INSPIRE-CNUM" in values)

    def test_date(self):
        """Test for proper date in 260."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="260",
                                    code="c"),
            ["2010"]
        )
        self.assertFalse(
            record_get_field_values(self.converted_record,
                                    tag="260",
                                    code="t")
        )


class TestINSPIRE2CDSConference(unittest.TestCase):

    """Test converted record data."""

    def setUp(self):
        """Load demo data."""
        from harvestingkit.bibrecord import BibRecordPackage
        from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS

        self.inspire_conf_demo_data_path = pkg_resources.resource_filename(
            'harvestingkit.tests',
            os.path.join('data', 'sample_inspire_conf.xml')
        )

        bibrecs = BibRecordPackage(self.inspire_conf_demo_data_path)
        bibrecs.parse()
        self.parsed_record = bibrecs.get_records()[0]
        self.package = Inspire2CDS(self.parsed_record)
        self.recid = self.package.get_recid()
        self.converted_record = self.package.get_record()

    def test_conference_tag(self):
        """Test for conference tag in 690C."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="690",
                                    ind1="C",
                                    code="a"),
            ["CONFERENCE"]
        )

    def test_cnum(self):
        """Make sure that CNUM is okay."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="035",
                                    code="a",
                                    filter_subfield_code="9",
                                    filter_subfield_value="Inspire-CNUM"),
            ["C16-03-21.3"]
        )

    def test_date(self):
        """Test for proper date in 260."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="260",
                                    code="c"),
            ["2016"]
        )

    def test_collection(self):
        """Test for proper collection in 980."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="980",
                                    code="a"),
            ["ANNOUNCEMENT"]
        )

    def test_conference_info(self):
        """Test for proper info in 111."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="111",
                                    code="a"),
            ["Workshop on Topics in Three Dimensional Gravity"]
        )
        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="111",
                                    code="9"),
            ["20160321"]
        )
        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="111",
                                    code="f"),
            ["2016"]
        )
        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="111",
                                    code="c"),
            ["Miramare, Trieste, Italy"]
        )
        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="111",
                                    code="g"),
            ["miramare20160321"]
        )
        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="111",
                                    code="d"),
            ["21-24 Mar 2016"]
        )

    def test_conference_info_date_parsing(self):
        """Test conversion with the special cases for dates."""
        from harvestingkit.bibrecord import record_get_field_values
        from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS

        xml = """<collection>
        <record>
          <datafield tag="111" ind1=" " ind2=" ">
            <subfield code="x">2016-03-21</subfield>
            <subfield code="y">2016-03-24</subfield>
            <subfield code="c">Somewhere, Someplace</subfield>
          </datafield>
          <datafield tag="111" ind1=" " ind2=" ">
            <subfield code="x">2016-03-30</subfield>
            <subfield code="y">2016-04-03</subfield>
            <subfield code="c">Somewhere</subfield>
          </datafield>
          <datafield tag="111" ind1=" " ind2=" ">
            <subfield code="x">2016-05-21</subfield>
            <subfield code="c">Someplace</subfield>
          </datafield>
          <datafield tag="111" ind1=" " ind2=" ">
            <subfield code="y">2016-03-21</subfield>
          </datafield>
          <datafield tag="111" ind1=" " ind2=" ">
            <subfield code="x">2016-03-24</subfield>
          </datafield>
          <datafield tag="980" ind1=" " ind2=" ">
            <subfield code="a">CONFERENCES</subfield>
          </datafield>
        </record>
        </collection>
        """
        for record in Inspire2CDS.from_source(xml):
            converted_record = record.get_record()
            fields_111_d = record_get_field_values(converted_record,
                                                   tag="111",
                                                   code="d")

            assert len(fields_111_d) == 2
            assert '21-24 Mar 2016' in fields_111_d
            assert '30 Mar-03 Apr 2016' in fields_111_d


            fields_111_g = record_get_field_values(converted_record,
                                                   tag="111",
                                                   code="g")
            assert len(fields_111_g) == 3
            assert 'somewhere20160330' in fields_111_g
            assert 'somewhere20160321' in fields_111_g
            assert 'someplace20160521' in fields_111_g


class TestINSPIRE2CDSConferencePaper(unittest.TestCase):

    """Test converted record data."""

    def setUp(self):
        """Load demo data."""
        from harvestingkit.bibrecord import BibRecordPackage
        from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS

        self.inspire_demo_data_path = pkg_resources.resource_filename(
            'harvestingkit.tests',
            os.path.join('data', 'sample_inspire_oai.xml')
        )

        bibrecs = BibRecordPackage(self.inspire_demo_data_path)
        bibrecs.parse()
        parsed_record = bibrecs.get_records()[1]
        self.package = Inspire2CDS(parsed_record)
        self.recid = self.package.get_recid()
        self.converted_record = self.package.get_record()

    def test_cnum(self):
        """Make sure that CNUM is okay."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertTrue(
            record_get_field_values(self.converted_record,
                                    tag="773",
                                    code="w",
                                    filter_subfield_code="w",
                                    filter_subfield_value="C96-05-21",
                                    filter_subfield_mode="e")
        )
        self.assertFalse(
            record_get_field_values(self.converted_record,
                                    tag="035",
                                    code="a",
                                    filter_subfield_code="9",
                                    filter_subfield_value="INSPIRE-CNUM",
                                    filter_subfield_mode="e")
        )

    def test_noinspirekeywords(self):
        """Make sure that no INSPIRE keywords are there."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertFalse(
            record_get_field_values(self.converted_record,
                                    tag="695",
                                    code="2",
                                    filter_subfield_code="2",
                                    filter_subfield_value="INSPIRE",
                                    filter_subfield_mode="e")
        )

    def test_collectionname(self):
        """Make sure that the right collection name is there."""
        from harvestingkit.bibrecord import record_get_field_values

        collections = record_get_field_values(
            self.converted_record,
            tag="980",
            code="a"
        )
        self.assertTrue("ConferencePaper" in collections)
        self.assertTrue("ARTICLE" in collections)
        self.assertTrue("PREPRINT" not in collections)

    def test_date(self):
        """Test for proper date in 260."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="260",
                                    code="c"),
            ["1997"]
        )

    def test_cern_tag(self):
        """Test for CERN tag in 690."""
        from harvestingkit.bibrecord import record_get_field_values

        self.assertEqual(
            record_get_field_values(self.converted_record,
                                    tag="690",
                                    ind1="C",
                                    code="a"),
            ["CERN"]
        )


class TestINSPIRE2CDSGeneric(unittest.TestCase):

    """Test specific conversions operations."""

    def test_experiments_conversion_with_leading_zero(self):
        """Test experiments conversion with removing leading zero."""
        from harvestingkit.bibrecord import record_get_field_values
        from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS

        xml = """<collection>
        <record>
        <datafield tag="693" ind1=" " ind2=" ">
            <subfield code="e">CERN-RD-053</subfield>
        </datafield>
        </record></collection>
        """
        for record in Inspire2CDS.from_source(xml):
            converted_record = record.get_record()

            self.assertEqual(
                record_get_field_values(converted_record,
                                        tag="693",
                                        code="a"),
                ["Not applicable"]
            )
            self.assertEqual(
                record_get_field_values(converted_record,
                                        tag="693",
                                        code="e"),
                ["RD53"]
            )

    def test_experiments_conversion_with_no_a(self):
        """Test experiments conversion with no $$a."""
        from harvestingkit.bibrecord import record_get_field_values
        from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS

        xml = """<collection>
        <record>
        <datafield tag="693" ind1=" " ind2=" ">
            <subfield code="e">CERN-CAST</subfield>
        </datafield>
        </record></collection>
        """
        for record in Inspire2CDS.from_source(xml):
            converted_record = record.get_record()

            self.assertEqual(
                record_get_field_values(converted_record,
                                        tag="693",
                                        code="a"),
                []
            )
            self.assertEqual(
                record_get_field_values(converted_record,
                                        tag="693",
                                        code="e"),
                ["CAST"]
            )

    def test_pubnote_conversion_with_pos_special_case(self):
        """Test pubnote conversion with the PoS special case for $p and $v."""
        from harvestingkit.bibrecord import record_get_field_values
        from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS

        xml = """<collection>
        <record>
          <datafield tag="773" ind1=" " ind2=" ">
            <subfield code="c">018</subfield>
            <subfield code="p">PoS</subfield>
            <subfield code="v">QFTHEP2011</subfield>
            <subfield code="w">C11-09-24</subfield>
            <subfield code="y">2013</subfield>
          </datafield>
        </record></collection>
        """
        for record in Inspire2CDS.from_source(xml):
            converted_record = record.get_record()

            self.assertEqual(
                record_get_field_values(converted_record,
                                        tag="773",
                                        code="p"),
                ["PoS"]
            )
            self.assertEqual(
                record_get_field_values(converted_record,
                                        tag="773",
                                        code="v"),
                ["QFTHEP2011"]
            )

    def test_author_conversion_with_no_v(self):
        """Test author conversion with the special case for $v."""
        from harvestingkit.bibrecord import record_get_field_values
        from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS

        xml = """<collection>
        <record>
        <datafield tag="100" ind1=" " ind2=" ">
          <subfield code="a">Mokhov, N.V.</subfield>
          <subfield code="v">Fermilab</subfield>
        </datafield>
        </record></collection>
        """
        for record in Inspire2CDS.from_source(xml):
            converted_record = record.get_record()

            self.assertEqual(
                record_get_field_values(converted_record,
                                        tag="100",
                                        code="v"),
                []
            )
            self.assertEqual(
                record_get_field_values(converted_record,
                                        tag="100",
                                        code="a"),
                ["Mokhov, N V"]
            )

    def test_link_conversion_with_no_w(self):
        """Test link conversion with the special case for $w."""
        from harvestingkit.bibrecord import record_get_field_values
        from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS

        xml = """<collection>
        <record>
          <datafield tag="856" ind1="4" ind2=" ">
            <subfield code="u">http://www.adsabs.harvard.edu/abs/1990NuPhS..13..535R</subfield>
            <subfield code="w">1990NuPhS..13..535R</subfield>
            <subfield code="y">ADSABS</subfield>
          </datafield>
        </record></collection>
        """
        for record in Inspire2CDS.from_source(xml):
            converted_record = record.get_record()

            self.assertEqual(
                record_get_field_values(converted_record,
                                        tag="856",
                                        ind1="4",
                                        code="w"),
                []
            )
            self.assertEqual(
                record_get_field_values(converted_record,
                                        tag="856",
                                        ind1="4",
                                        code="u"),
                ["http://www.adsabs.harvard.edu/abs/1990NuPhS..13..535R"]
            )

    def test_thesis_conversion(self):
        """Test link conversion with the special case for $w."""
        from harvestingkit.bibrecord import record_get_field_values
        from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS

        xml = """<collection>
        <record>
          <datafield tag="502" ind1=" " ind2=" ">
            <subfield code="b">Diploma</subfield>
            <subfield code="c">Freiburg U.</subfield>
            <subfield code="d">2005</subfield>
          </datafield>
          <datafield tag="980" ind1=" " ind2=" ">
            <subfield code="a">THESIS</subfield>
          </datafield>
        </record></collection>
        """
        for record in Inspire2CDS.from_source(xml):
            converted_record = record.get_record()

            self.assertEqual(
                record_get_field_values(converted_record,
                                        tag="980",
                                        code="a"),
                ["THESIS"]
            )

            self.assertEqual(
                record_get_field_values(converted_record,
                                        tag="502",
                                        code="a"),
                ["Diploma"]
            )
            self.assertEqual(
                record_get_field_values(converted_record,
                                        tag="502",
                                        code="b"),
                ["Freiburg U."]
            )
            self.assertEqual(
                record_get_field_values(converted_record,
                                        tag="502",
                                        code="c"),
                ["2005"]
            )

    @unittest.skip("Mapping tag 701 -> 700 disabled")
    def test_thesis_conversion_supervisors(self):
        """Test link conversion with the special case for $w."""
        from harvestingkit.bibrecord import record_get_field_values
        from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS

        xml = """<collection>
        <record>
          <datafield tag="701" ind1=" " ind2=" ">
            <subfield code="a">Besançon, Marc</subfield>
          </datafield>
          <datafield tag="701" ind1=" " ind2=" ">
            <subfield code="a">Ferri, Frederico</subfield>
          </datafield>
          <datafield tag="980" ind1=" " ind2=" ">
            <subfield code="a">THESIS</subfield>
          </datafield>
        </record></collection>
        """
        for record in Inspire2CDS.from_source(xml):
            converted_record = record.get_record()

            self.assertEqual(
                record_get_field_values(converted_record,
                                        tag="980",
                                        code="a"),
                ["THESIS"]
            )

            self.assertEqual(
                record_get_field_values(converted_record,
                                        tag="700",
                                        code="a"),
                ["Besançon, Marc", "Ferri, Frederico"]
            )
            self.assertEqual(
                record_get_field_values(converted_record,
                                        tag="700",
                                        code="e"),
                ["dir.", "dir."]
            )

    def test_journal_mappings(self):
        """Test journal mappings"""
        from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS
        self.assertEqual(Inspire2CDS.get_config_item('Nuovo Cim.', 'journals'), 'Nuovo Cimento')

    def test_add_collaboration_to_710g(self):
        """Append ' Collaboration' string to the collaboration name."""
        from harvestingkit.bibrecord import record_get_field_values
        from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS

        xml = """
        <collection>
            <record>
                <datafield tag="710" ind1=" " ind2=" ">
                    <subfield code="g">ATLAS</subfield>
                </datafield>
                <datafield tag="710" ind1=" " ind2=" ">
                    <subfield code="g">CMS Collaboration</subfield>
                </datafield>
                <datafield tag="800" ind1=" " ind2=" ">
                    <subfield code="g">another field</subfield>
                </datafield>
            </record>
        </collection>
        """

        for record in Inspire2CDS.from_source(xml):
            converted_record = record.get_record()

            tags_710g = record_get_field_values(converted_record,
                                                tag="710",
                                                code="g")

            self.assertEqual(tags_710g, ["ATLAS Collaboration", "CMS Collaboration"])

    def test_article_to_publication(self):
        """Test 542__e -> 542__3 Article is converted to Publication."""
        from harvestingkit.bibrecord import field_get_subfield_instances,\
            record_get_field_values, record_get_field_instances
        from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS

        xml = """
        <collection>
            <record>
                <datafield tag="542" ind1=" " ind2=" ">
                    <subfield code="g">Another Field</subfield>
                    <subfield code="e">Not Article</subfield>
                    <subfield code="a">Another Field</subfield>
                </datafield>
                <datafield tag="542" ind1=" " ind2=" ">
                    <subfield code="a">Another Field</subfield>
                    <subfield code="e">Article</subfield>
                </datafield>
                <datafield tag="542" ind1=" " ind2=" ">
                    <subfield code="a">Only `a` subfield</subfield>
                </datafield>
            </record>
            <record>
                <datafield tag="542" ind1=" " ind2=" ">
                    <subfield code="e">AnotherValue</subfield>
                </datafield>
            </record>
        </collection>
        """
        rec1, rec2 = Inspire2CDS.from_source(xml)

        converted_rec1 = rec1.get_record()

        rec1_542_fields = record_get_field_instances(converted_rec1, '542')
        rec1_542_0_subs = field_get_subfield_instances(rec1_542_fields[0])
        self.assertEqual(
            rec1_542_0_subs,
            [('g', 'Another Field'), ('3', 'Not Article'), ('a', 'Another Field')])

        rec1_542_1_subs = field_get_subfield_instances(rec1_542_fields[1])
        self.assertEqual(
            rec1_542_1_subs, [('a', 'Another Field'), ('3', 'publication')])

        rec1_542_2_subs = field_get_subfield_instances(rec1_542_fields[2])
        self.assertEqual(
            rec1_542_2_subs, [('a', 'Only `a` subfield')])

        converted_rec2 = rec2.get_record()

        rec2_542_fields = record_get_field_instances(converted_rec2, '542')
        rec2_542_0_subs = field_get_subfield_instances(rec2_542_fields[0])
        self.assertEqual(
            rec2_542_0_subs, [('3', 'AnotherValue')])


    def test_article_773(self):
        """Test if tag 773 has c,p,v,y then doc_type is ARTICLE."""
        from harvestingkit.bibrecord import record_get_field_values
        from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS

        xml = """
        <collection>
            <record>
                <datafield tag="773" ind1=" " ind2=" ">
                    <subfield code="x">Phys. Rev. D 91 (2015) 021302 (Rapid Communication)</subfield>
                    <subfield code="v">D91</subfield>
                    <subfield code="p">Phys.Rev.</subfield>
                    <subfield code="y">2015</subfield>
                    <subfield code="c">021302</subfield>
                </datafield>
                <datafield tag="980" ind1=" " ind2=" ">
                    <subfield code="a">PUBLISHED</subfield>
                </datafield>
            </record>
            <record>
                <datafield tag="773" ind1=" " ind2=" ">
                    <subfield code="x">Phys. Rev. D 91 (2015) 021302 (Rapid Communication)</subfield>
                    <subfield code="v">D91</subfield>
                    <subfield code="p">Phys.Rev.</subfield>
                    <subfield code="y">2015</subfield>
                    <subfield code="c">021302</subfield>
                </datafield>
                <datafield tag="980" ind1=" " ind2=" ">
                    <subfield code="a">CONFERENCEPAPER</subfield>
                </datafield>
            </record>
            <record>
                <datafield tag="773" ind1=" " ind2=" ">
                    <subfield code="w">C10-09-06.10</subfield>
                </datafield>
            </record>
        </collection>
        """

        records = list(Inspire2CDS.from_source(xml))

        rec1 = records[0]
        converted_record = rec1.get_record()
        tag_980 = record_get_field_values(converted_record, tag="980", code="a")
        self.assertEqual(tag_980, ["ARTICLE"])

        rec2 = records[1]
        converted_record = rec2.get_record()
        tag_980 = record_get_field_values(converted_record, tag="980", code="a")
        self.assertEqual(sorted(tag_980), sorted(["ConferencePaper", "ARTICLE"]))

        rec3 = records[2]
        converted_record = rec3.get_record()
        tag_980 = record_get_field_values(converted_record, tag="980", code="a")
        self.assertEqual(tag_980, ["PREPRINT"])

    def test_ignore_999(self):
        """Test ignore tag 999."""
        from harvestingkit.bibrecord import record_get_field_values
        from harvestingkit.inspire_cds_package.from_inspire import Inspire2CDS

        xml = """
        <collection>
            <record>
                <datafield tag="999" ind1="C" ind2="5">
                    <subfield code="h">I. Krajcar Bronić, B. Grosswendt Nuclear</subfield>
                    <subfield code="m">Instruments and Methods in Physics Research Section B 142 , p. 219 Article | PDF (618 K) | View Record in Scopus | Citing articles (12)</subfield>
                    <subfield code="o">16</subfield>
                    <subfield code="y">1998</subfield>
                </datafield>
                <datafield tag="999" ind1=" " ind2=" ">
                    <subfield code="o">16</subfield>
                    <subfield code="y">1998</subfield>
                </datafield>
                <datafield tag="999" ind1=" " ind2="3">
                    <subfield code="o">16</subfield>
                    <subfield code="y">1998</subfield>
                </datafield>
            </record>
        </collection>
        """

        for record in Inspire2CDS.from_source(xml):
            converted_record = record.get_record()

            tag_999 = record_get_field_values(converted_record, tag="999")
            self.assertEqual(tag_999, [])


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
##
## This file is part of Harvesting Kit.
## Copyright (C) 2014 CERN.
##
## Harvesting Kit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License as
## published by the Free Software Foundation; either version 2 of the
## License, or (at your option) any later version.
##
## Harvesting Kit is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Harvesting Kit; if not, write to the Free Software Foundation, Inc.,
## 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
from __future__ import print_function
import sys
import re

from xml.dom.minidom import parse
from harvestingkit.utils import (fix_journal_name,
                                 format_arxiv_id)
from harvestingkit.bibrecord import (
    record_add_field,
    create_record,
    record_xml_output,
)
from harvestingkit.minidom_utils import (get_value_in_tag,
                                         xml_to_text,
                                         get_all_text)
from harvestingkit.jats_package import JatsPackage


class ApsPackageXMLError(Exception):

    """Raised when the XML given is of the wrong format."""


class ApsPackage(JatsPackage):

    """Parse an APS harvested file in JATS format and creating a MARCXML."""

    def __init__(self, journal_mappings=None):
        super(ApsPackage, self).__init__(journal_mappings)

    def _get_reference(self, ref):
        """Retrieve the data for a reference."""
        label = get_value_in_tag(ref, 'label')
        label = re.sub('\D', '', label)
        for innerref in ref.getElementsByTagName('mixed-citation'):
            ref_type = innerref.getAttribute('publication-type')
            institution = get_value_in_tag(innerref, 'institution')
            report_no = ''
            for tag in innerref.getElementsByTagName('pub-id'):
                if tag.getAttribute('pub-id-type') == 'other':
                    if tag.hasChildNodes():
                        report_no = get_all_text(tag)
            doi = ''
            for tag in innerref.getElementsByTagName('pub-id'):
                if tag.getAttribute('pub-id-type') == 'doi':
                    doi = xml_to_text(tag)
            collaboration = get_value_in_tag(innerref, 'collab')
            authors = []
            person_groups = innerref.getElementsByTagName('person-group')
            for author_group in person_groups:
                if author_group.getAttribute('person-group-type') == 'author':
                    for author in author_group.getElementsByTagName('string-name'):
                        if author.hasChildNodes():
                            authors.append(get_all_text(author))
            editors = []
            for editor_group in person_groups:
                if editor_group.getAttribute('person-group-type') == 'editor':
                    for editor in editor_group.getElementsByTagName('string-name'):
                        if editor.hasChildNodes():
                            editors.append(get_all_text(editor))
            journal = get_value_in_tag(innerref, 'source')
            journal, volume = fix_journal_name(journal, self.journal_mappings)
            volume += get_value_in_tag(innerref, 'volume')
            if journal == 'J.High Energy Phys.' or journal == 'JHEP':
                issue = get_value_in_tag(innerref, 'issue')
                volume = volume[2:] + issue
                journal = 'JHEP'
            page = get_value_in_tag(innerref, 'page-range')
            year = get_value_in_tag(innerref, 'year')
            external_link = get_value_in_tag(innerref, 'ext-link')
            arxiv = ''
            for tag in innerref.getElementsByTagName('pub-id'):
                if tag.getAttribute('pub-id-type') == 'arxiv':
                    if tag.hasChildNodes():
                        arxiv = get_all_text(tag)
            arxiv = format_arxiv_id(arxiv)
            publisher = get_value_in_tag(innerref, 'publisher-name')
            publisher_location = get_value_in_tag(innerref, 'publisher-loc')
            if publisher_location:
                publisher = publisher_location + ': ' + publisher
            unstructured_text = []
            for child in innerref.childNodes:
                if child.nodeType == child.TEXT_NODE:
                    text = child.nodeValue.strip()
                    text = re.sub(r'[\[\]\(\.;\)]', '', text).strip()
                    if text.startswith(','):
                        text = text[1:].strip()
                    if text.endswith('Report No'):
                        text = institution + " " + text
                        institution = ''
                        text = text.strip()
                    elif text.endswith(' ed'):
                        text += '.'
                    elif text.endswith('PhD thesis,'):
                        if institution:
                            text += ' ' + institution
                            institution = ''
                        else:
                            text = text[:-1]
                    elif text.startswith('Seminar,'):
                        article_title = get_value_in_tag(innerref, 'article-title')
                        text = institution + " Seminar, \"" + article_title + "\""
                        institution = ''
                    elif text == u'\u201d':
                        text = ''
                    ignore_text = ['in', 'pp', 'edited by']
                    if text.startswith('Vol'):
                        temp = re.sub(r'\D', '', text)
                        if temp:
                            volume += temp
                    elif len(text) > 1 and text not in ignore_text\
                            and not (text.isdigit() or text[:-1].isdigit()):
                        unstructured_text.append(text)
            if unstructured_text:
                unstructured_text = " ".join(unstructured_text)
            if ref_type == 'book':
                if volume and not volume.lower().startswith('vol'):
                    volume = 'Vol ' + volume
                if volume and page:
                    volume = volume + ', pp ' + page
            yield ref_type, doi, authors, collaboration, journal, volume, page, year,\
                label, arxiv, publisher, institution, unstructured_text, external_link,\
                report_no, editors

    def _add_references(self, rec):
        """ Adds the reference to the record """
        for ref in self.document.getElementsByTagName('ref'):
            for ref_type, doi, authors, collaboration, journal, volume, page, year,\
                    label, arxiv, publisher, institution, unstructured_text,\
                    external_link, report_no, editors in self._get_reference(ref):
                subfields = []
                if doi:
                    subfields.append(('a', doi))
                for author in authors:
                    subfields.append(('h', author))
                for editor in editors:
                    subfields.append(('e', editor))
                if year:
                    subfields.append(('y', year))
                if unstructured_text:
                    if page:
                        subfields.append(('m', unstructured_text + ', ' + page))
                    else:
                        subfields.append(('m', unstructured_text))
                if collaboration:
                    subfields.append(('c', collaboration))
                if institution:
                    subfields.append(('m', institution))
                if publisher:
                    subfields.append(('p', publisher))
                if arxiv:
                    subfields.append(('r', arxiv))
                if report_no:
                    subfields.append(('r', report_no))
                if external_link:
                    subfields.append(('u', external_link))
                if label:
                    subfields.append(('o', label))
                if ref_type == 'book':
                    if journal:
                        subfields.append(('t', journal))
                    if volume:
                        subfields.append(('m', volume))
                    elif page and not unstructured_text:
                        subfields.append(('m', page))
                else:
                    if volume and page:
                        subfields.append(('s', journal + "," + volume + "," + page))
                    elif journal:
                        subfields.append(('t', journal))
                if ref_type:
                    subfields.append(('d', ref_type))
                if not subfields:
                    #misc-type references
                    try:
                        r = ref.getElementsByTagName('mixed-citation')[0]
                        text = xml_to_text(r)
                        label = text.split()[0]
                        text = " ".join(text.split()[1:])
                        subfields.append(('s', text))
                        record_add_field(rec, '999', ind1='C', ind2='5', subfields=subfields)
                    except IndexError:
                        #references without 'mixed-citation' tag
                        try:
                            r = ref.getElementsByTagName('note')[0]
                            subfields.append(('s', xml_to_text(r)))
                            record_add_field(rec, '999', ind1='C', ind2='5', subfields=subfields)
                        except IndexError:
                            #references without 'note' tag
                            subfields.append(('s', xml_to_text(ref)))
                            record_add_field(rec, '999', ind1='C', ind2='5', subfields=subfields)
                else:
                    record_add_field(rec, '999', ind1='C', ind2='5', subfields=subfields)

    def get_record(self, xml_file):
        """ Reads a xml file in JATS format and returns
            a xml string in marc format """
        self.document = parse(xml_file)

        if get_value_in_tag(self.document, "meta"):
            raise ApsPackageXMLError("The XML format of %s is not correct"
                                     % (xml_file,))
        page_count = self._get_page_count()
        rec = create_record()
        if page_count:
            record_add_field(rec, '300', subfields=[('a', page_count)])
        pacscodes = self._get_pacscodes()
        for pacscode in pacscodes:
            record_add_field(rec, '084', subfields=[('2', 'PACS'),
                                                    ('a', pacscode)])
        subject = self._get_subject()
        if subject:
            record_add_field(rec, '650', ind1='1', ind2='7', subfields=[('2', 'APS'),
                                                                        ('a', subject)])
        keywords = self._get_keywords()
        if keywords:
            record_add_field(rec, '653', ind1='1', subfields=[('a', ', '.join(keywords)),
                                                              ('9', 'author')])
        title, subtitle, _ = self._get_title()
        subfields = []
        if subtitle:
            subfields.append(('b', subtitle))
        if title:
            subfields.append(('a', title))
            record_add_field(rec, '245', subfields=subfields)
        journal, volume, issue, year, start_date, doi,\
            article_id, _, _ = self._get_publication_information()
        if start_date:
            record_add_field(rec, '260', subfields=[('c', start_date),
                                                    ('t', 'published')])
        if doi:
            record_add_field(rec, '024', ind1='7', subfields=[('a', doi),
                                                              ('2', 'DOI')])
        abstract = self._get_abstract()
        if abstract:
            record_add_field(rec, '520', subfields=[('a', abstract),
                                                    ('9', 'APS')])
        license, license_type, license_url = self._get_license()
        subfields = []
        if license:
            subfields.append(('a', license))
        if license_url:
            subfields.append(('u', license_url))
        if subfields:
            record_add_field(rec, '540', subfields=subfields)
        c_holder, c_year, c_statement = self._get_copyright()
        c_holder, c_year, c_statement = self._get_copyright()
        if c_holder and c_year:
            record_add_field(rec, '542', subfields=[('d', c_holder),
                                                    ('g', c_year),
                                                    ('e', 'Article')])
        elif c_statement:
            record_add_field(rec, '542', subfields=[('f', c_statement),
                                                    ('e', 'Article')])
        record_add_field(rec, '773', subfields=[('p', journal),
                                                ('v', volume),
                                                ('n', issue),
                                                ('y', year),
                                                ('c', article_id)])
        record_add_field(rec, '980', subfields=[('a', 'HEP')])
        record_add_field(rec, '980', subfields=[('a', 'Citeable')])
        record_add_field(rec, '980', subfields=[('a', 'Published')])
        self._add_authors(rec)
        self._add_references(rec)
        try:
            return record_xml_output(rec)
        except UnicodeDecodeError:
            sys.stderr.write("""Found a bad char in the file
                                for the article """ + doi)
            return ""


def main(args):
    if len(args) != 1:
        print("usage: python aps_package.py <filename>")
        sys.exit()
    a = ApsPackage()
    print(a.get_record(args[0]))

if __name__ == '__main__':
    main(sys.argv[1:])
